@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * SMSC Connection interface for Bearerbox.
  *
@@ -27,6 +83,7 @@
 #include "bearerbox.h"
 #include "numhash.h"
 #include "smscconn.h"
+#include "dlr.h"
 
 #include "bb_smscconn_cb.h"    /* callback functions for connections */
 #include "smscconn_p.h"        /* to access counters */
@@ -34,7 +91,6 @@
 /* passed from bearerbox core */
 
 extern volatile sig_atomic_t bb_status;
-extern List *incoming_wdp;
 extern List *incoming_sms;
 extern List *outgoing_sms;
 
@@ -49,43 +105,29 @@ extern List *isolated;
 
 static volatile sig_atomic_t smsc_running;
 static List *smsc_list;
+static RWLock smsc_list_lock;
 static List *smsc_groups;
 static Octstr *unified_prefix;
 
 static Numhash *black_list;
 static Numhash *white_list;
 
-static long maximum_queue_length;
+static regex_t *white_list_regex;
+static regex_t *black_list_regex;
 
 static long router_thread = -1;
 
-static void log_sms(SMSCConn *conn, Msg *sms, char *message)
-{
-    Octstr *text, *udh;
-    
-    text = sms->sms.msgdata ? octstr_duplicate(sms->sms.msgdata) : octstr_create("");
-    udh = sms->sms.udhdata ? octstr_duplicate(sms->sms.udhdata) : octstr_create("");
-    if ((sms->sms.coding == DC_8BIT || sms->sms.coding == DC_UCS2))
-	octstr_binary_to_hex(text, 1);
-    octstr_binary_to_hex(udh, 1);
+/*
+ * Counter for catenated SMS messages. The counter that can be put into
+ * the catenated SMS message's UDH headers is actually the lowest 8 bits.
+ */
+Counter *split_msg_counter;
+
+/*
+ * forward declaration
+ */
+static long route_incoming_to_smsc(SMSCConn *conn, Msg *msg);
 
-    alog("%s [SMSC:%s] [SVC:%s] [ACT:%s] [from:%s] [to:%s] [flags:%d:%d:%d:%d:%d] [msg:%d:%s]"
-	" [udh:%d:%s]",
-	 message,
-	 conn ? (smscconn_id(conn) ? octstr_get_cstr(smscconn_id(conn)) : "")
-	 : "",
-	 sms->sms.service ? octstr_get_cstr(sms->sms.service) : "",
-	 sms->sms.account ? octstr_get_cstr(sms->sms.account) : "",
-	 sms->sms.sender ? octstr_get_cstr(sms->sms.sender) : "",
-	 sms->sms.receiver ? octstr_get_cstr(sms->sms.receiver) : "",
-	 sms->sms.mclass, sms->sms.coding, sms->sms.mwi, sms->sms.compress,
-	 sms->sms.dlr_mask,
-	 octstr_len(sms->sms.msgdata), octstr_get_cstr(text),
-	 octstr_len(sms->sms.udhdata), octstr_get_cstr(udh)
-    );
-    octstr_destroy(udh);
-    octstr_destroy(text);
-}
 
 /*---------------------------------------------------------------------------
  * CALLBACK FUNCTIONS
@@ -117,33 +159,99 @@ void bb_smscconn_killed(void)
 }
 
 
-void bb_smscconn_sent(SMSCConn *conn, Msg *sms)
+static void handle_split(SMSCConn *conn, Msg *msg, long reason)
 {
-    Msg *mack;
+    struct split_parts *split = msg->sms.split_parts;
     
-    counter_increase(outgoing_sms_counter);
-    if (conn) counter_increase(conn->sent);
+    /* if temporarely failed, try again immediately */
+    if (reason == SMSCCONN_FAILED_TEMPORARILY && smscconn_send(conn, msg) == 0)
+        return;
     
-    /* write ACK to store file */
+    /*
+     * if the reason is not a success and status is still success
+     * then set status of a split to the reason.
+     * Note: reason 'malformed','discarded' or 'rejected' has higher priority!
+     */
+    switch(reason) {
+    case SMSCCONN_FAILED_DISCARDED:
+    case SMSCCONN_FAILED_REJECTED:
+    case SMSCCONN_FAILED_MALFORMED:
+        debug("bb.sms.splits", 0, "Set split msg status to %ld", reason);
+        split->status = reason;
+        break;
+    case SMSCCONN_SUCCESS:
+        break; /* nothing todo */
+    default:
+        if (split->status == SMSCCONN_SUCCESS) {
+            debug("bb.sms.splits", 0, "Set split msg status to %ld", reason);
+            split->status = reason;
+        }
+        break;
+    }
 
-    mack = msg_create(ack);
-    mack->ack.nack = 0;
-    mack->ack.time = sms->sms.time;
-    mack->ack.id = sms->sms.id;
-    
-    (void) store_save(mack);
-    msg_destroy(mack);
-    
-    /* XXX relay confirmancy message should be generated here */
-    
-    log_sms(conn, sms, "Sent SMS");
-    msg_destroy(sms);
+    /*
+     * now destroy this message, because we don't need it anymore.
+     * we will split it again in smscconn_send(...).
+     */
+    msg_destroy(msg);
+        
+    if (counter_decrease(split->parts_left) <= 1) {
+        /* all splited parts were processed */
+        counter_destroy(split->parts_left);
+        msg = split->orig;
+        msg->sms.split_parts = NULL;
+        if (split->status == SMSCCONN_SUCCESS)
+            bb_smscconn_sent(conn, msg, NULL);
+        else {
+            debug("bb.sms.splits", 0, "Parts of concatenated message failed.");
+            bb_smscconn_send_failed(conn, msg, split->status, NULL);
+        }
+        gw_free(split);
+    }
 }
 
 
-void bb_smscconn_send_failed(SMSCConn *conn, Msg *sms, int reason)
+void bb_smscconn_sent(SMSCConn *conn, Msg *sms, Octstr *reply)
 {
-    Msg *mnack;
+    if (sms->sms.split_parts != NULL) {
+        handle_split(conn, sms, SMSCCONN_SUCCESS);
+        return;
+    }
+    
+    counter_increase(outgoing_sms_counter);
+    if (conn) counter_increase(conn->sent);
+
+    /* write ACK to store file */
+    store_save_ack(sms, ack_success);
+
+    bb_alog_sms(conn, sms, "Sent SMS");
+
+    /* generate relay confirmancy message */
+    if (DLR_IS_SMSC_SUCCESS(sms->sms.dlr_mask)) {
+        Msg *dlrmsg;
+
+	if (reply == NULL)
+	    reply = octstr_create("");
+
+	octstr_insert_data(reply, 0, "ACK/", 4);
+        dlrmsg = create_dlr_from_msg((conn->id?conn->id:conn->name), sms,
+	                reply, DLR_SMSC_SUCCESS);
+        if (dlrmsg != NULL) {
+            bb_smscconn_receive(conn, dlrmsg);
+        }
+    }
+
+    msg_destroy(sms);
+    octstr_destroy(reply);
+}
+
+
+void bb_smscconn_send_failed(SMSCConn *conn, Msg *sms, int reason, Octstr *reply)
+{
+    if (sms->sms.split_parts != NULL) {
+        handle_split(conn, sms, reason);
+        return;
+    }
     
     switch (reason) {
 
@@ -152,90 +260,134 @@ void bb_smscconn_send_failed(SMSCConn *conn, Msg *sms, int reason)
 	list_produce(outgoing_sms, sms);
 	break;
     default:
-
 	/* write NACK to store file */
+        store_save_ack(sms, ack_failed);
 
-	mnack = msg_create(ack);
-	mnack->ack.nack = 1;
-	mnack->ack.time = sms->sms.time;
-	mnack->ack.id = sms->sms.id;
-
-	(void) store_save(mnack);
-	msg_destroy(mnack);
-    
 	if (conn) counter_increase(conn->failed);
 	if (reason == SMSCCONN_FAILED_DISCARDED)
-	    log_sms(conn, sms, "DISCARDED SMS");
+	    bb_alog_sms(conn, sms, "DISCARDED SMS");
 	else
-	    log_sms(conn, sms, "FAILED Send SMS");
-	msg_destroy(sms);
-    } 
-}    
+	    bb_alog_sms(conn, sms, "FAILED Send SMS");
 
-int bb_smscconn_receive(SMSCConn *conn, Msg *sms)
+        /* generate relay confirmancy message */
+        if (DLR_IS_SMSC_FAIL(sms->sms.dlr_mask) ||
+	    DLR_IS_FAIL(sms->sms.dlr_mask)) {
+            Msg *dlrmsg;
+
+	    if (reply == NULL)
+	        reply = octstr_create("");
+
+	    octstr_insert_data(reply, 0, "NACK/", 5);
+            dlrmsg = create_dlr_from_msg((conn ? (conn->id?conn->id:conn->name) : NULL), sms,
+	                                 reply, DLR_SMSC_FAIL);
+            if (dlrmsg != NULL) {
+                bb_smscconn_receive(conn, dlrmsg);
+            }
+        }
+	msg_destroy(sms);
+    }
+
+    octstr_destroy(reply);
+}
+
+long bb_smscconn_receive(SMSCConn *conn, Msg *sms)
 {
     char *uf;
+    int rc;
+    Msg *copy;
 
-    /* do some queue control */
-    if (maximum_queue_length != -1 && bb_status == BB_FULL && 
-            list_len(incoming_sms) <= maximum_queue_length) {
-        bb_status = BB_RUNNING;
-        warning(0, "started to accept messages again");
-    }
-    
-    if (maximum_queue_length != -1 && 
-            list_len(incoming_sms) > maximum_queue_length) {
-        if (bb_status != BB_FULL)
-            bb_status = BB_FULL;
-        warning(0, "incoming messages queue too long, dropping a message");
-        log_sms(conn, sms, "DROPPED Received SMS");
-        gwthread_sleep(0.1); /* letting the queue go down */
-     	return -1;
-    }
-
-    /* else if (list_len(incoming_sms) > 100)
-     *	gwthread_sleep(0.5);
-     */
-    
-    if (unified_prefix == NULL)
-    	uf = NULL;
-    else
-    	uf = octstr_get_cstr(unified_prefix);
+   /*
+    * First normalize in smsc level and then on global level.
+    * In outbound direction it's vise versa, hence first global then smsc.
+    */
+    uf = (conn && conn->unified_prefix) ? octstr_get_cstr(conn->unified_prefix) : NULL;
+    normalize_number(uf, &(sms->sms.sender));
 
+    uf = unified_prefix ? octstr_get_cstr(unified_prefix) : NULL;
     normalize_number(uf, &(sms->sms.sender));
 
     if (white_list &&
 	numhash_find_number(white_list, sms->sms.sender) < 1) {
 	info(0, "Number <%s> is not in white-list, message discarded",
 	     octstr_get_cstr(sms->sms.sender));
-	log_sms(conn, sms, "REJECTED - not white-listed SMS");
+	bb_alog_sms(conn, sms, "REJECTED - not white-listed SMS");
 	msg_destroy(sms);
-        return -1;
+        return SMSCCONN_FAILED_REJECTED;
     }
+
+    if (white_list_regex && (gw_regex_matches(white_list_regex, sms->sms.sender) == NO_MATCH)) {
+        info(0, "Number <%s> is not in white-list, message discarded",
+             octstr_get_cstr(sms->sms.sender));
+        bb_alog_sms(conn, sms, "REJECTED - not white-regex-listed SMS");
+        msg_destroy(sms);
+        return SMSCCONN_FAILED_REJECTED;
+    }
+    
     if (black_list &&
 	numhash_find_number(black_list, sms->sms.sender) == 1) {
 	info(0, "Number <%s> is in black-list, message discarded",
 	     octstr_get_cstr(sms->sms.sender));
-	log_sms(conn, sms, "REJECTED - black-listed SMS");
+	bb_alog_sms(conn, sms, "REJECTED - black-listed SMS");
 	msg_destroy(sms);
-	return -1;
+	return SMSCCONN_FAILED_REJECTED;
     }
 
-    if (sms->sms.sms_type != report)
+    if (black_list_regex && (gw_regex_matches(black_list_regex, sms->sms.sender) == NO_MATCH)) {
+        info(0, "Number <%s> is not in black-list, message discarded",
+             octstr_get_cstr(sms->sms.sender));
+        bb_alog_sms(conn, sms, "REJECTED - black-regex-listed SMS");
+        msg_destroy(sms);
+        return SMSCCONN_FAILED_REJECTED;
+    }
+
+    if (sms->sms.sms_type != report_mo)
 	sms->sms.sms_type = mo;
-    
+
     /* write to store (if enabled) */
     if (store_save(sms) == -1)
-	return -1;
-    
-    if (sms->sms.sms_type != report)
-	log_sms(conn, sms, "Receive SMS");
+	return SMSCCONN_FAILED_TEMPORARILY;
+
+    copy = msg_duplicate(sms);
+
+    /*
+     * Try to reroute internally to an smsc-id without leaving
+     * actually bearerbox scope.
+     * Scope: internal routing (to smsc-ids)
+     */
+    if ((rc = route_incoming_to_smsc(conn, copy)) == -1) {
+        /*
+         * Now try to route the message to a specific smsbox
+         * connection based on the existing msg->sms.boxc_id or
+         * the registered receiver numbers for specific smsbox'es.
+         * Scope: external routing (to smsbox connections)
+         */
+        if (route_incoming_to_boxc(copy) == -1) {
+            warning(0, "incoming messages queue too long, dropping a message.");
+            if (sms->sms.sms_type == report_mo)
+                bb_alog_sms(conn, sms, "DROPPED Received DLR");
+            else
+                bb_alog_sms(conn, sms, "DROPPED Received SMS");
+
+            /* put nack into store-file */
+            store_save_ack(sms, ack_failed);
+
+            msg_destroy(copy);
+
+            msg_destroy(sms);
+            gwthread_sleep(0.1); /* letting the queue go down */
+            return SMSCCONN_FAILED_QFULL;
+        }
+    }
+
+    if (sms->sms.sms_type != report_mo)
+	bb_alog_sms(conn, sms, "Receive SMS");
     else
-	log_sms(conn, sms, "DLR SMS");
+	bb_alog_sms(conn, sms, "DLR SMS");
 
-    list_produce(incoming_sms, sms);
     counter_increase(incoming_sms_counter);
-    counter_increase(conn->received);
+    if (conn != NULL) counter_increase(conn->received);
+
+    msg_destroy(sms);
 
     return 0;
 }
@@ -246,7 +398,7 @@ int bb_smscconn_receive(SMSCConn *conn, Msg *sms)
  */
 
 
-    
+
 /* function to route outgoing SMS'es from delay-list
  * use some nice magics to route them to proper SMSC
  */
@@ -286,15 +438,16 @@ static void sms_router(void *arg)
 	ret = smsc2_rout(msg);
 	if (ret == -1) {
             warning(0, "No SMSCes to receive message, discarding it!");
-	    bb_smscconn_send_failed(NULL, msg, SMSCCONN_FAILED_DISCARDED);
+	    bb_smscconn_send_failed(NULL, msg, SMSCCONN_FAILED_DISCARDED,
+	                            octstr_create("DISCARDED"));
         } else if (ret == 1) {
 	    newmsg = startmsg = NULL;
 	}
-	    
-	    
+
+
     }
     /* router has died, make sure that rest die, too */
-    
+
     smsc_running = 0;
 
     list_remove_producer(flow_threads);
@@ -317,14 +470,14 @@ int smsc2_start(Cfg *cfg)
 
     if (smsc_running) return -1;
 
+    /* create split sms counter */
+    split_msg_counter = counter_create();
+    
     smsc_list = list_create();
-    smsc_groups = list_create();
+    gw_rwlock_init_static(&smsc_list_lock);
 
     grp = cfg_get_single_group(cfg, octstr_imm("core"));
     unified_prefix = cfg_get(grp, octstr_imm("unified-prefix"));
-    if (cfg_get_integer(&maximum_queue_length, grp, 
-                           octstr_imm("maximum-queue-length")) == -1)
-        maximum_queue_length = -1;
 
     white_list = black_list = NULL;
     os = cfg_get(grp, octstr_imm("white-list"));
@@ -332,11 +485,22 @@ int smsc2_start(Cfg *cfg)
         white_list = numhash_create(octstr_get_cstr(os));
 	octstr_destroy(os);
     }
+    if ((os = cfg_get(grp, octstr_imm("white-list-regex"))) != NULL) {
+        if ((white_list_regex = gw_regex_comp(os, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(os));
+        octstr_destroy(os);
+    }
+    
     os = cfg_get(grp, octstr_imm("black-list"));
     if (os != NULL) {
         black_list = numhash_create(octstr_get_cstr(os));
 	octstr_destroy(os);
     }
+    if ((os = cfg_get(grp, octstr_imm("black-list-regex"))) != NULL) {
+        if ((black_list_regex = gw_regex_comp(os, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(os));
+        octstr_destroy(os);
+    }
 
     smsc_groups = cfg_get_multi_group(cfg, octstr_imm("smsc"));
     /*
@@ -362,24 +526,28 @@ int smsc2_start(Cfg *cfg)
 	panic(0, "Failed to start a new thread for SMS routing");
     
     list_add_producer(incoming_sms);
-    list_add_producer(incoming_wdp);
     smsc_running = 1;
     return 0;
 }
 
-static int smsc2_find(Octstr *id)
+/*
+ * Find a matching smsc-id in the smsc list starting at position start.
+ * NOTE: Caller must ensure that smsc_list is properly locked!
+ */
+static long smsc2_find(Octstr *id, long start)
 {
     SMSCConn *conn = NULL;
-    int i;
+    long i;
 
-    list_lock(smsc_list);
-    for (i = 0; i < list_len(smsc_list); i++) {
+    if (start > list_len(smsc_list) || start < 0)
+        return -1;
+
+    for (i = start; i < list_len(smsc_list); i++) {
         conn = list_get(smsc_list, i);
         if (conn != NULL && octstr_compare(conn->id, id) == 0) {
             break;
         }
     }
-    list_unlock(smsc_list);
     if (i >= list_len(smsc_list))
         i = -1;
     return i;
@@ -388,121 +556,151 @@ static int smsc2_find(Octstr *id)
 int smsc2_stop_smsc(Octstr *id)
 {
     SMSCConn *conn;
-    int i;
+    long i = -1;
 
+    if (!smsc_running)
+        return -1;
+
+    gw_rwlock_rdlock(&smsc_list_lock);
     /* find the specific smsc via id */
-    if ((i = smsc2_find(id)) == -1) {
-        error(0, "HTTP: Could not shutdown smsc-id `%s'", octstr_get_cstr(id));
-        return -1;
+    while((i = smsc2_find(id, ++i)) != -1) {
+        conn = list_get(smsc_list, i);
+        if (conn != NULL && smscconn_status(conn) == SMSCCONN_DEAD) {
+            info(0, "HTTP: Could not shutdown already dead smsc-id `%s'",
+                octstr_get_cstr(id));
+        } else {
+            info(0,"HTTP: Shutting down smsc-id `%s'", octstr_get_cstr(id));
+            smscconn_shutdown(conn, 1);   /* shutdown the smsc */
+        }
     }
-    conn = list_get(smsc_list, i);
-    if (conn != NULL && conn->status == SMSCCONN_DEAD) {
-        error(0, "HTTP: Could not shutdown already deaed smsc-id `%s'", 
-              octstr_get_cstr(id));
-        return -1;
-    }
-    info(0,"HTTP: Shutting down smsc-id `%s'", octstr_get_cstr(id));
-    smscconn_shutdown(conn, 1);   /* shutdown the smsc */
+    gw_rwlock_unlock(&smsc_list_lock);
     return 0;
 }
 
 int smsc2_restart_smsc(Octstr *id)
 {
     CfgGroup *grp;
-    SMSCConn *conn;
-    Octstr *smscid;
-    int i;
+    SMSCConn *conn, *new_conn;
+    Octstr *smscid = NULL;
+    long i = -1;
+    int num = 0;
 
+    if (!smsc_running)
+        return -1;
+
+    gw_rwlock_wrlock(&smsc_list_lock);
     /* find the specific smsc via id */
-    if ((i = smsc2_find(id)) == -1) {
-        error(0, "HTTP: Could not re-start non defined smsc-id `%s'", 
-              octstr_get_cstr(id));
-        return -1;
-    }
-
-    /* check if smsc is online status already */
-    conn = list_get(smsc_list, i);
-    if (conn != NULL && conn->status != SMSCCONN_DEAD) {
-        error(0, "HTTP: Could not re-start already running smsc-id `%s'", 
-              octstr_get_cstr(id));
-        return -1;
-    }
-  
-    list_delete(smsc_list, i, 1); /* drop it from the active smsc list */
-	smscconn_destroy(conn);       /* destroy the connection */
-
-    /* find the group with smsc id */
-    grp = NULL;
-    list_lock(smsc_groups);
-    for (i = 0; i < list_len(smsc_groups) && 
-        (grp = list_get(smsc_groups, i)) != NULL; i++) {
-        smscid = cfg_get(grp, octstr_imm("smsc-id"));
-        if (smscid != NULL && octstr_compare(smscid, id) == 0) {
+    while((i = smsc2_find(id, ++i)) != -1) {
+        int hit;
+        long group_index;
+        /* check if smsc has online status already */
+        conn = list_get(smsc_list, i);
+        if (conn != NULL && smscconn_status(conn) != SMSCCONN_DEAD) {
+            warning(0, "HTTP: Could not re-start already running smsc-id `%s'",
+                octstr_get_cstr(id));
+            continue;
+        }
+        /* find the group with equal smsc id */
+        hit = 0;
+        grp = NULL;
+        for (group_index = 0; group_index < list_len(smsc_groups) && 
+             (grp = list_get(smsc_groups, group_index)) != NULL; group_index++) {
+            smscid = cfg_get(grp, octstr_imm("smsc-id"));
+            if (smscid != NULL && octstr_compare(smscid, id) == 0) {
+                if (hit == num)
+                    break;
+                else
+                    hit++;
+            }
+            octstr_destroy(smscid);
+            smscid = NULL;
+        }
+        octstr_destroy(smscid);
+        if (hit != num) {
+            /* config group not found */
+            error(0, "HTTP: Could not find config for smsc-id `%s'", octstr_get_cstr(id));
             break;
         }
+        
+        info(0,"HTTP: Re-starting smsc-id `%s'", octstr_get_cstr(id));
+
+        new_conn = smscconn_create(grp, 1);
+        if (new_conn == NULL) {
+            error(0, "Start of SMSC connection failed, smsc-id `%s'", octstr_get_cstr(id));
+            continue; /* keep old connection on the list */
+        }
+        
+        /* drop old connection from the active smsc list */
+        list_delete(smsc_list, i, 1);
+        /* destroy the connection */
+        smscconn_destroy(conn);
+        list_insert(smsc_list, i, new_conn);
+        smscconn_start(new_conn);
+        num++;
     }
-    list_unlock(smsc_groups);
-    if (i > list_len(smsc_groups))
-        return -1;
+    gw_rwlock_unlock(&smsc_list_lock);
     
-    info(0,"HTTP: Re-starting smsc-id `%s'", octstr_get_cstr(id));
-
-    conn = smscconn_create(grp, 1); 
-    if (conn == NULL)
-        error(0, "Cannot start with SMSC connection failing");
-    list_append(smsc_list, conn);
-
-    smscconn_start(conn);
+    /* wake-up the router */
     if (router_thread >= 0)
         gwthread_wakeup(router_thread);
-
+    
     return 0;
 }
 
 void smsc2_resume(void)
 {
     SMSCConn *conn;
-    int i;
-    
+    long i;
+
+    if (!smsc_running)
+        return;
+
+    gw_rwlock_rdlock(&smsc_list_lock);
     for (i = 0; i < list_len(smsc_list); i++) {
         conn = list_get(smsc_list, i);
         smscconn_start(conn);
     }
+    gw_rwlock_unlock(&smsc_list_lock);
+    
     if (router_thread >= 0)
-	gwthread_wakeup(router_thread);
+        gwthread_wakeup(router_thread);
 }
 
 
 void smsc2_suspend(void)
 {
     SMSCConn *conn;
-    int i;
-    
-    list_lock(smsc_list);
+    long i;
+
+    if (!smsc_running)
+        return;
+
+    gw_rwlock_rdlock(&smsc_list_lock);
     for (i = 0; i < list_len(smsc_list); i++) {
         conn = list_get(smsc_list, i);
         smscconn_stop(conn);
     }
-    list_unlock(smsc_list);
+    gw_rwlock_unlock(&smsc_list_lock);
 }
 
 
 int smsc2_shutdown(void)
 {
     SMSCConn *conn;
-    int i;
+    long i;
 
-    if (!smsc_running) return -1;
+    if (!smsc_running)
+        return -1;
 
     /* Call shutdown for all SMSC Connections; they should
      * handle that they quit, by emptying queues and then dying off
      */
-    list_lock(smsc_list);
+    gw_rwlock_rdlock(&smsc_list_lock);
     for(i=0; i < list_len(smsc_list); i++) {
         conn = list_get(smsc_list, i);
 	smscconn_shutdown(conn, 1);
     }
-    list_unlock(smsc_list);
+    gw_rwlock_unlock(&smsc_list_lock);
     if (router_thread >= 0)
 	gwthread_wakeup(router_thread);
 
@@ -513,7 +711,6 @@ int smsc2_shutdown(void)
      * to shutdown before calling these?
      */
     list_remove_producer(incoming_sms);
-    list_remove_producer(incoming_wdp);
     return 0;
 }
 
@@ -521,21 +718,32 @@ int smsc2_shutdown(void)
 void smsc2_cleanup(void)
 {
     SMSCConn *conn;
-    int i;
+    long i;
+
+    if (!smsc_running)
+        return;
 
     debug("smscconn", 0, "final clean-up for SMSCConn");
     
-    list_lock(smsc_list);
+    gw_rwlock_wrlock(&smsc_list_lock);
     for (i = 0; i < list_len(smsc_list); i++) {
         conn = list_get(smsc_list, i);
         smscconn_destroy(conn);
     }
-    list_unlock(smsc_list);
     list_destroy(smsc_list, NULL);
+    smsc_list = NULL;
+    gw_rwlock_unlock(&smsc_list_lock);
     list_destroy(smsc_groups, NULL);
     octstr_destroy(unified_prefix);    
     numhash_destroy(white_list);
     numhash_destroy(black_list);
+    if (white_list_regex != NULL)
+        gw_regex_destroy(white_list_regex);
+    if (black_list_regex != NULL)
+        gw_regex_destroy(black_list_regex);
+    /* destroy msg split counter */
+    counter_destroy(split_msg_counter);
+    gw_rwlock_destroy(&smsc_list_lock);
 }
 
 
@@ -544,11 +752,12 @@ Octstr *smsc2_status(int status_type)
     Octstr *tmp;
     char tmp3[64];
     char *lb;
-    int i, para = 0;
+    long i;
+    int para = 0;
     SMSCConn *conn;
     StatusInfo info;
-    Octstr *conn_id = NULL;
-    Octstr *conn_name = NULL;
+    const Octstr *conn_id = NULL;
+    const Octstr *conn_name = NULL;
 
     if ((lb = bb_status_linebreak(status_type)) == NULL)
         return octstr_create("Un-supported format");
@@ -568,8 +777,8 @@ Octstr *smsc2_status(int status_type)
         tmp = octstr_format("%sSMSC connections:%s", para ? "<p>" : "", lb);
     else
         tmp = octstr_format("<smscs><count>%d</count>\n\t", list_len(smsc_list));
-    
-    list_lock(smsc_list);
+
+    gw_rwlock_rdlock(&smsc_list_lock);
     for (i = 0; i < list_len(smsc_list); i++) {
         conn = list_get(smsc_list, i);
 
@@ -637,7 +846,7 @@ Octstr *smsc2_status(int status_type)
             info.received, info.sent, info.failed,
             info.queued, lb);
     }
-    list_unlock(smsc_list);
+    gw_rwlock_unlock(&smsc_list_lock);
 
     if (para)
         octstr_append_cstr(tmp, "</p>");
@@ -671,11 +880,6 @@ int smsc2_rout(Msg *msg)
     if (msg_type(msg) != sms)
 	return -1;
     
-    if (list_len(smsc_list) == 0) {
-	warning(0, "No SMSCes to receive message");
-	return -1;
-    }
-
     /* unify prefix of receiver, in case of it has not been
      * already done */
 
@@ -685,8 +889,12 @@ int smsc2_rout(Msg *msg)
     /* select in which list to add this
      * start - from random SMSCConn, as they are all 'equal'
      */
-
-    list_lock(smsc_list);
+    gw_rwlock_rdlock(&smsc_list_lock);
+    if (list_len(smsc_list) == 0) {
+	warning(0, "No SMSCes to receive message");
+        gw_rwlock_unlock(&smsc_list_lock);
+	return SMSCCONN_FAILED_DISCARDED;
+    }
 
     s = gw_rand() % list_len(smsc_list);
     best_preferred = best_ok = NULL;
@@ -722,7 +930,6 @@ int smsc2_rout(Msg *msg)
 	    bo_load = info.load;
 	}
     }
-    list_unlock(smsc_list);
 
     if (best_preferred)
 	ret = smscconn_send(best_preferred, msg);
@@ -731,15 +938,18 @@ int smsc2_rout(Msg *msg)
     else if (bad_found) {
 	if (bb_status != BB_SHUTDOWN)
 	    list_produce(outgoing_sms, msg);
+        gw_rwlock_unlock(&smsc_list_lock);
 	return 0;
     }
     else {
+        gw_rwlock_unlock(&smsc_list_lock);
 	if (bb_status == BB_SHUTDOWN)
 	    return 0;
 	warning(0, "Cannot find SMSCConn for message to <%s>, rejected.",
 		octstr_get_cstr(msg->sms.receiver));
 	return -1;
     }
+    gw_rwlock_unlock(&smsc_list_lock);
     /* check the status of sending operation */
     if (ret == -1)
 	return (smsc2_rout(msg));	/* re-try */
@@ -749,15 +959,59 @@ int smsc2_rout(Msg *msg)
 }
 
 
+/*
+ * Try to reroute to another smsc.
+ * @return -1 if no rerouting info available; otherwise return code from smsc2_route.
+ */
+static long route_incoming_to_smsc(SMSCConn *conn, Msg *msg)
+{
+    Octstr *smsc;
+    
+    /* sanity check */
+    if (!conn || !msg)
+        return -1;
+        
+    /* check for dlr rerouting */
+    if (!conn->reroute_dlr && (msg->sms.sms_type == report_mo || msg->sms.sms_type == report_mt))
+        return -1;
 
+    /*
+     * Check if we have any "reroute" rules to obey. Which means msg gets
+     * transported internally from MO to MT msg.
+     */
+    if (conn->reroute) {
+        /* change message direction */
+        store_save_ack(msg, ack_success);
+        msg->sms.sms_type = mt_push;
+        store_save(msg);
+        /* drop into outbound queue again for routing */
+        return smsc2_rout(msg);
+    }
+    
+    if (conn->reroute_to_smsc) {
+        /* change message direction */
+        store_save_ack(msg, ack_success);
+        msg->sms.sms_type = mt_push;
+        store_save(msg);
+        /* apply directly to the given smsc-id for MT traffic */
+        octstr_destroy(msg->sms.smsc_id);
+        msg->sms.smsc_id = octstr_duplicate(conn->reroute_to_smsc);
+        return smsc2_rout(msg);
+    }
+    
+    if (conn->reroute_by_receiver && msg->sms.receiver &&
+                 (smsc = dict_get(conn->reroute_by_receiver, msg->sms.receiver))) {
+        /* change message direction */
+        store_save_ack(msg, ack_success);
+        msg->sms.sms_type = mt_push;
+        store_save(msg);
+        /* route by receiver number */
+        /* XXX implement wildcard matching too! */
+        octstr_destroy(msg->sms.smsc_id);
+        msg->sms.smsc_id = octstr_duplicate(smsc);
+        return smsc2_rout(msg);
+    }
 
-
-
-
-
-
-
-
-
-
+    return -1; 
+}
 
