@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * bearerbox.c
  * 
@@ -35,6 +91,9 @@ Counter *outgoing_sms_counter;
 Counter *incoming_wdp_counter;
 Counter *outgoing_wdp_counter;
 
+/* incoming/outgoing sms queue control */
+long max_incoming_sms_qlength;
+
 
 
 /* this is not a list of items; instead it is used as
@@ -60,12 +119,21 @@ List *isolated;
 
 volatile sig_atomic_t bb_status;
 
+/* 
+ * Flags for main thread to check what is to do.
+ */
+enum {
+    BB_LOGREOPEN = 1,
+    BB_CHECKLEAKS = 2
+};
+/* Here we will set above flags */
+static volatile sig_atomic_t bb_todo = 0;
+
 /* own global variables */
 
 static Mutex *status_mutex;
-Mutex *boxid_mutex;
 static time_t start_time;
-
+volatile sig_atomic_t restart = 0;
 
 
 /* to avoid copied code */
@@ -99,29 +167,19 @@ static void signal_handler(int signum)
     switch (signum) {
         case SIGINT:
         case SIGTERM:
-
-            mutex_lock(status_mutex);
             if (bb_status != BB_SHUTDOWN && bb_status != BB_DEAD) {
-                warning(0, "Killing signal received, shutting down...");
-                mutex_unlock(status_mutex);
-                bb_shutdown();
-                return;
-            } 
+                bb_status = BB_SHUTDOWN;
+            }
             else if (bb_status == BB_SHUTDOWN) {
-                warning(0, "New killing signal received, killing neverthless...");
                 bb_status = BB_DEAD;
             }
             else if (bb_status == BB_DEAD) {
-                panic(0, "cannot die by its own will");
+                panic(0, "Cannot die by its own will");
             }
-            mutex_unlock(status_mutex);
             break;
 
         case SIGHUP:
-            warning(0, "SIGHUP received, catching and re-opening logs");
-            log_reopen();
-            alog_reopen();
-            store_load();
+            bb_todo |= BB_LOGREOPEN;
             break;
         
         /* 
@@ -129,9 +187,8 @@ static void signal_handler(int signum)
          * platforms that's reserved by the pthread support. 
          */
         case SIGQUIT:
-	       warning(0, "SIGQUIT received, reporting memory usage.");
-	       gw_check_leaks();
-	       break;
+           bb_todo |= BB_CHECKLEAKS;  
+           break;
     }
 }
 
@@ -188,14 +245,16 @@ static void wdp_router(void *arg)
 
 	gw_assert(msg_type(msg) == wdp_datagram);
 	
-	// if (msg->list == sms)
-	// smsc_addwdp(msg);
-	// else
+	/*
+	if (msg->list == sms)
+	    smsc_addwdp(msg);
+	else
+	*/
 
 	udp_addwdp(msg);
     }
     udp_die();
-    // smsc_endwdp();
+    /* smsc_endwdp(); */
 
     list_remove_producer(flow_threads);
 }
@@ -247,7 +306,7 @@ static int check_config(Cfg *cfg)
     if (cfg_get_integer(&wapp, grp, octstr_imm("wapbox-port")) == -1)
     	wapp = -1;
     
-#ifndef KANNEL_NO_SMS    
+#ifndef NO_SMS    
     grp = cfg_get_single_group(cfg, octstr_imm("smsbox"));
     if (smsp != -1 && grp == NULL) {
 	error(0, "No 'smsbox' group in configuration, but smsbox-port set");
@@ -255,7 +314,7 @@ static int check_config(Cfg *cfg)
     }
 #endif
     
-#ifndef KANNEL_NO_WAP	
+#ifndef NO_WAP	
     grp = cfg_get_single_group(cfg, octstr_imm("wapbox"));
     if (wapp != -1 && grp == NULL) {
 	error(0, "No 'wapbox' group in configuration, but wapbox-port set");
@@ -276,10 +335,10 @@ static int check_args(int i, int argc, char **argv) {
     else if (strcmp(argv[i], "-I")==0 || strcmp(argv[i], "--isolated")==0)
 	bb_status = BB_ISOLATED;
     else
-	return -1;
+        return -1;
 
     return 0;
-} 
+}
 
 
 static Cfg *init_bearerbox(Cfg *cfg)
@@ -287,12 +346,15 @@ static Cfg *init_bearerbox(Cfg *cfg)
     CfgGroup *grp;
     Octstr *log, *val;
     long loglevel;
+    int lf, m;
 #ifdef HAVE_LIBSSL
     Octstr *ssl_server_cert_file;
     Octstr *ssl_server_key_file;
     int ssl_enabled = 0;
 #endif /* HAVE_LIBSSL */
- 
+
+    /* defaults: use localtime and markers for access-log */
+    lf = m = 1;
 	
     grp = cfg_get_single_group(cfg, octstr_imm("core"));
 
@@ -300,25 +362,39 @@ static Cfg *init_bearerbox(Cfg *cfg)
     if (log != NULL) {
 	if (cfg_get_integer(&loglevel, grp, octstr_imm("log-level")) == -1)
 	    loglevel = 0;
-	log_open(octstr_get_cstr(log), loglevel);
+	log_open(octstr_get_cstr(log), loglevel, GW_NON_EXCL);
 	octstr_destroy(log);
     }
 
     if (check_config(cfg) == -1)
-	panic(0, "Cannot start with corrupted configuration");
+        panic(0, "Cannot start with corrupted configuration");
 
-    log = cfg_get(grp, octstr_imm("access-log"));
-    if (log != NULL) {
-	alog_open(octstr_get_cstr(log), 1);
-	/* use localtime; XXX let user choose that */
+    /* determine which timezone we use for access logging */
+    if ((log = cfg_get(grp, octstr_imm("access-log-time"))) != NULL) {
+        lf = (octstr_case_compare(log, octstr_imm("gmt")) == 0) ? 0 : 1;
+        octstr_destroy(log);
+    }
 
-	octstr_destroy(log);
+    /* should predefined markers be used, ie. prefixing timestamp */
+    cfg_get_bool(&m, grp, octstr_imm("access-log-clean"));
+
+    /* custom access-log format  */
+    if ((log = cfg_get(grp, octstr_imm("access-log-format"))) != NULL) {
+        bb_alog_init(log);
+        octstr_destroy(log);
+    }
+
+    /* open access-log file */
+    if ((log = cfg_get(grp, octstr_imm("access-log"))) != NULL) {
+        alog_open(octstr_get_cstr(log), lf, m ? 0 : 1);
+        octstr_destroy(log);
     }
 
     log = cfg_get(grp, octstr_imm("store-file"));
+    /* initialize the store file */
     if (log != NULL) {
-	store_init(log);
-	octstr_destroy(log);
+        store_init(log);
+        octstr_destroy(log);
     }
 
     conn_config_ssl (grp);
@@ -359,7 +435,6 @@ static Cfg *init_bearerbox(Cfg *cfg)
     incoming_wdp_counter = counter_create();
     
     status_mutex = mutex_create();
-    boxid_mutex = mutex_create();
 
     setup_signal_handlers();
 
@@ -367,7 +442,20 @@ static Cfg *init_bearerbox(Cfg *cfg)
     /* http-admin is REQUIRED */
     httpadmin_start(cfg);
 
-#ifndef KANNEL_NO_SMS    
+    if (cfg_get_integer(&max_incoming_sms_qlength, grp,
+                           octstr_imm("maximum-queue-length")) == -1)
+        max_incoming_sms_qlength = -1;
+    else {
+        warning(0, "Option 'maximum-queue-length' is deprecated! Please use"
+                          " 'sms-incoming-queue-limit' instead!");
+    }
+
+    if (max_incoming_sms_qlength == -1 &&
+        cfg_get_integer(&max_incoming_sms_qlength, grp,
+                                  octstr_imm("sms-incoming-queue-limit")) == -1)
+        max_incoming_sms_qlength = -1;
+
+#ifndef NO_SMS    
     {
 	List *list;
 	
@@ -379,7 +467,7 @@ static Cfg *init_bearerbox(Cfg *cfg)
     }
 #endif
     
-#ifndef KANNEL_NO_WAP
+#ifndef NO_WAP
     grp = cfg_get_single_group(cfg, octstr_imm("core"));
     val = cfg_get(grp, octstr_imm("wdp-interface-name"));
     if (val != NULL && octstr_len(val) > 0)
@@ -398,7 +486,7 @@ static void empty_msg_lists(void)
 {
     Msg *msg;
 
-#ifndef KANNEL_NO_WAP
+#ifndef NO_WAP
 
     if (list_len(incoming_wdp) > 0 || list_len(outgoing_wdp) > 0)
 	warning(0, "Remaining WDP: %ld incoming, %ld outgoing",
@@ -421,7 +509,7 @@ static void empty_msg_lists(void)
     counter_destroy(outgoing_wdp_counter);
     
     
-#ifndef KANNEL_NO_SMS
+#ifndef NO_SMS
 
     /* XXX we should record these so that they are not forever lost...
      */
@@ -481,7 +569,7 @@ int main(int argc, char **argv)
     init_bearerbox(cfg);
 
     info(0, "----------------------------------------");
-    info(0, GW_NAME " bearerbox II version %s starting", VERSION);
+    info(0, GW_NAME " bearerbox II version %s starting", GW_VERSION);
 
 
     gwthread_sleep(5.0); /* give time to threads to register themselves */
@@ -501,8 +589,43 @@ int main(int argc, char **argv)
 	list_remove_producer(isolated);
     }
 
-    /* wait until flow threads exit */
+    while(bb_status != BB_SHUTDOWN && bb_status != BB_DEAD && list_producer_count(flow_threads) > 0) {
+        /* debug("bb", 0, "Main Thread: going to sleep."); */
+        /*
+         * Not infinite sleep here, because we should notice
+         * when all "flow threads" are dead and shutting bearerbox
+         * down.
+         * XXX if all "flow threads" call gwthread_wakeup(MAIN_THREAD_ID),
+         * we can enter infinite sleep then.
+         */
+        gwthread_sleep(10.0);
+        /* debug("bb", 0, "Main Thread: woken up."); */
 
+        if (bb_todo == 0) {
+            continue;
+        }
+
+        if (bb_todo & BB_LOGREOPEN) {
+            warning(0, "SIGHUP received, catching and re-opening logs");
+            log_reopen();
+            alog_reopen();
+            bb_todo = bb_todo & ~BB_LOGREOPEN;
+        }
+
+        if (bb_todo & BB_CHECKLEAKS) {
+            warning(0, "SIGQUIT received, reporting memory usage.");
+	    gw_check_leaks();
+            bb_todo = bb_todo & ~BB_CHECKLEAKS;
+        }
+    }
+
+    if (bb_status == BB_SHUTDOWN || bb_status == BB_DEAD)
+        warning(0, "Killing signal or HTTP admin command received, shutting down...");
+
+    /* call shutdown */
+    bb_shutdown();
+
+    /* wait until flow threads exit */
     while(list_consume(flow_threads)!=NULL)
 	;
 
@@ -517,13 +640,16 @@ int main(int argc, char **argv)
     list_destroy(suspended, NULL);
     list_destroy(isolated, NULL);
     mutex_destroy(status_mutex);
-    mutex_destroy(boxid_mutex);
 
     alog_close();		/* if we have any */
+    bb_alog_shutdown();
     cfg_destroy(cfg);
     dlr_shutdown();
     gwlib_shutdown();
 
+    if (restart == 1)
+        execvp(argv[0],argv);
+
     return 0;
 }
 
@@ -547,17 +673,17 @@ int bb_shutdown(void)
 
     called = 1;
     set_shutdown_status();
-    store_shutdown();
     mutex_unlock(status_mutex);
 
-#ifndef KANNEL_NO_SMS
+#ifndef NO_SMS
     debug("bb", 0, "shutting down smsc");
     smsc2_shutdown();
 #endif
-#ifndef KANNEL_NO_WAP
+#ifndef NO_WAP
     debug("bb", 0, "shutting down udp");
     udp_shutdown();
 #endif
+    store_shutdown();
     
     return 0;
 }
@@ -636,7 +762,8 @@ int bb_restart_smsc(Octstr *id)
 
 int bb_restart(void)
 {
-    return -1;
+    restart = 1;
+    return bb_shutdown();
 }
 
 
@@ -728,7 +855,7 @@ Octstr *bb_print_status(int status_type)
         (float)counter_value(incoming_sms_counter)/t,
         (float)counter_value(outgoing_sms_counter)/t,
         dlr_messages(),
-        octstr_get_cstr(dlr_type));
+        dlr_type());
 
     octstr_destroy(version);
     ret = octstr_create(buf);
