@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*****************************************************************************
  * smsc_ois.c - Sema Group SMS2000 (G6.0) Center (OIS 5.0).
  * Jouko Koski (EDS) for WapIT Ltd.
@@ -922,7 +978,7 @@ static int ois_append_submission_options(char *raw, const Msg *msg)
     if (octstr_len(msg->sms.udhdata)) {
 	raw[0] |= (char) 0x02;
     }
-    if (msg->sms.coding == DC_8BIT) { /* XXX UCS2? */
+    if (msg->sms.coding == DC_8BIT) { /* XXX and UCS2? */
 	raw[0] |= (char) 0x10;
     }
     return 1;
@@ -937,7 +993,7 @@ static int ois_append_sm_text(char *raw, const Msg *msg)
 
     SAY(3, "ois_append_sm_text");
 
-    if (msg->sms.coding == DC_7BIT) {
+    if (msg->sms.coding == DC_7BIT || msg->sms.coding == DC_UNDEF) {
         charset_latin1_to_gsm(msg->sms.udhdata);
         charset_latin1_to_gsm(msg->sms.msgdata);
     }
@@ -1175,8 +1231,8 @@ static int ois_adjust_data_coding_scheme(Msg *msg, const char *raw)
 {
     SAY(3, "ois_adjust_data_coding_scheme");
 
-    /* we set the value only temporarily: */
-    /* ois_adjust_sm_text will set the correct value */
+    /* we're using this variable temporarily: 
+     * ois_adjust_sm_text will set the correct value */
 
     msg->sms.coding = (raw[0] & 0xff) + 1;
 
@@ -1198,10 +1254,8 @@ static int ois_adjust_additional_information(Msg *msg, const char *raw)
 {
     SAY(3, "ois_adjust_additional_information");
 
-    /* we set the value only temporarily: */
-    /* ois_adjust_sm_text will set the correct value */
-
-    /* XXX I used mc temporarily. use fields_to_dcs! */
+    /* we're using this variable temporarily: 
+     * ois_adjust_sm_text will set the correct value */
     msg->sms.mclass = raw[0] & 0xff;
 
     return 1;
@@ -1221,7 +1275,7 @@ static int ois_adjust_sm_text(Msg *msg, const char *raw)
 
     /* copy text, note: flag contains temporarily the raw type description */
 
-    switch ((msg->sms.coding - 1) & 0xff) {
+    switch ((msg->sms.coding - 1) & 0xff) { 
     case 0x00: /* gsm7 */
 	ois_expand_gsm7(buffer, &raw[2], msglen7);
 	ois_convert_to_iso88591(buffer, msglen7);
