@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 
  /*
  * smsc_sema.c - implement sms2000 protocal by using X25 access
@@ -173,7 +229,7 @@ int sema_submit_msg(SMSCenter *smsc, Msg *msg)
 	    goto error;
 	}
 	/*  user data header is not supported in sm2000 X25 access
-	    if(msg->sms.coding == DC_7BIT){
+	    if(msg->sms.coding == DC_7BIT ...|| DC_UNDEF?){
 	    error(0, "sema_submit_sms: submit invoke support in IA5 encoding(8 bits chars)");
 	    goto error;
 	    }
@@ -231,7 +287,7 @@ int sema_submit_msg(SMSCenter *smsc, Msg *msg)
 	
 	memset(submit_sm->smscrefnum,0,sizeof(submit_sm->smscrefnum));	
         /*pack the message body in 2kmsg*/
-	lmsg->msgbody = (void**)(&submit_sm);
+	lmsg->msgbody = submit_sm;
 	nret = sema_msg_session_mt(smsc, lmsg);
 
 	gw_free(submit_sm);
@@ -245,11 +301,11 @@ int sema_submit_msg(SMSCenter *smsc, Msg *msg)
 	    return 1; /*success*/
 	}
 	else if(nret == SESSION_MT_RECEIVE_TIMEOUT){
-	    info(0, "sema_submit msg: session time out without return");
+	    info(0, "sema_submit msg: session timed out without return");
 	    return 0;
 	}
 	else if(nret == SESSION_MT_RECEIVE_ERR){
-	    info(0, "sema_submit msg: smsc say submit failed!");
+	    info(0, "sema_submit msg: smsc says submit failed!");
 	    return 0;
 	}
 
@@ -275,7 +331,7 @@ int sema_receive_msg(SMSCenter *smsc, Msg **msg)
 	*msg = msg_create(sms);
 	if(*msg==NULL) goto error;
     
-	recieve_sm = (struct sm_deliver_invoke*) (*(rmsg->msgbody));
+	recieve_sm = (struct sm_deliver_invoke*) rmsg->msgbody;
 	if(recieve_sm==NULL) goto error;
           
 	/* as IA5(8 bit character) is the default line encoding used by X28
@@ -468,7 +524,7 @@ static int X28_open_data_link(char* device){
 	error(errno, "sema_open_data_link: error open(2)ing the character device <%s>",
 	      device);
 	if(errno == EACCES)
-	    error(0, "sema_open_data_link: user have no right to access the serial device");
+	    error(0, "sema_open_data_link: user has no right to access the serial device");
 	return -1;
     }
 
@@ -635,14 +691,14 @@ static int X28_open_send_link(int padfd, char *nua) {
 	   readbuff[readall-2] == '\r') {
 	    if(strstr(readbuff, smscbuff)) {
 		debug("smsc.sema", 0,
-		      "sema_open send link: smsc respond, virtual link established");
+		      "sema_open send link: smsc responded, virtual link established");
 		x28_data_mode = X28_MT_DATA_MODE; 
 		return 1;	
 	    }
 	} 
 	usleep(1000);
     }
-    info(0,"sema_open_send_link: connecting time out");
+    info(0,"sema_open_send_link: connect timeout");
     return 0;
 error_overflow:
     warning(0, "sema_open_send_link: command buffer overflow");
@@ -741,7 +797,7 @@ static int X28_data_send(int padfd, char *cbuffer,int sentonce) {
     return writeall;
 
 error:
-    error(errno,"sema_send data error: device file erro");
+    error(errno,"sema_send data error: device file error");
     return -1;
 }
 
@@ -826,7 +882,7 @@ static int X28_msg_pop(char *from, char *to)
 	    memset(from,0,len);
     }
 
-    /* check rest line for link state: command mode or data mode */
+    /* check rest of line for link state: command mode or data mode */
     if(strstr(from,prompbuff) != NULL)
 	x28_data_mode = X28_COMMAND_MODE;
 
@@ -897,7 +953,7 @@ static int sema_msg_session_mt(SMSCenter *smsc, sema_msg* pmsg){
     int isrcved = 0, iTrcved = 0, decoderesult = 0;
     time_t tstart;
 
-    submit_invoke=(struct sm_submit_invoke*)(*(pmsg->msgbody));
+    submit_invoke = (struct sm_submit_invoke*) pmsg->msgbody;
     if(submit_invoke == NULL) goto error;
 
     /*encode first*/
@@ -998,7 +1054,7 @@ static int sema_msg_session_mt(SMSCenter *smsc, sema_msg* pmsg){
 		if(decoderesult >= 0){
 		    if(mtrmsg->type == 's'){ /*submit result*/
 	 
-			submit_result = (struct sm_submit_result*)(*mtrmsg->msgbody);
+			submit_result = (struct sm_submit_result*) mtrmsg->msgbody;
 			if(submit_result == NULL) goto error;
 			/* check result operation number is what we send */
 			memset(tmp1,0,5); memset(tmp2,0,5);
@@ -1018,7 +1074,7 @@ static int sema_msg_session_mt(SMSCenter *smsc, sema_msg* pmsg){
 		    } 
 		    else if(mtrmsg->type == 'T'){ /*report invoke*/
 		
-			report_invoke=(struct sm_statusreport_invoke*)(*mtrmsg->msgbody);
+			report_invoke = (struct sm_statusreport_invoke*) mtrmsg->msgbody;
 			if(report_invoke == NULL) goto error;
 			/*check if report reference number is what we expect*/
 			memset(tmp1,0,sizeof(tmp1)); memset(tmp2,0,sizeof(tmp2));
@@ -1031,7 +1087,7 @@ static int sema_msg_session_mt(SMSCenter *smsc, sema_msg* pmsg){
 			iret = sema_submit_result(smsc, mtrmsg, decoderesult);
 			if(iret == -1) goto error;
 			if(iTrcved == 1 &&
-			   report_invoke->status != 3){ /*3 means ,msg delivered*/
+			   report_invoke->status != 3){ /*3 means msg delivered*/
 			    info(0,"sema_mt_session: submit invoke failed with report value-%i",report_invoke->status);
 			    gw_free(report_invoke);
 			    goto smsc_say_fail;
@@ -1045,7 +1101,7 @@ static int sema_msg_session_mt(SMSCenter *smsc, sema_msg* pmsg){
 			decoderesult = 0;
 			iret = sema_submit_result(smsc, mtrmsg, decoderesult);
 			if(iret == -1) goto error;
-			deliver_invoke =(struct sm_deliver_invoke*)(*mtrmsg->msgbody);
+			deliver_invoke = (struct sm_deliver_invoke*) mtrmsg->msgbody;
 			if(deliver_invoke != NULL){
 			    gw_free(deliver_invoke);
 			    /*append buffer back to  smsc->buffer*/
@@ -1063,13 +1119,13 @@ static int sema_msg_session_mt(SMSCenter *smsc, sema_msg* pmsg){
 		memset(data,0,sizeof(data));
 		if(sema_wait_report == 0 && isrcved == 1)
 		{
-		    info(0,"sema_mt_session: submit invoke deliver successfully to smsc");
+		    info(0,"sema_mt_session: submit invoke delivered successfully to smsc");
 		    goto mo_success;
 		}
 		if(sema_wait_report > 0 &&
 		   isrcved == 1 && iTrcved == 1)
 		{
-		    info(0,"sema_mt_session: submit invoke deliver sucessfully to msisdn");
+		    info(0,"sema_mt_session: submit invoke delivered successfully to msisdn");
 		    goto mo_success;
 		}
 	    }
@@ -1077,7 +1133,7 @@ static int sema_msg_session_mt(SMSCenter *smsc, sema_msg* pmsg){
     }
 
 /* mo_timeout: */
-      info(0,"sema_mt_session: time out without receive all expected return");
+      info(0,"sema_mt_session: timeout without receiving all expected returns");
       moret = SESSION_MT_RECEIVE_TIMEOUT;
       goto mo_return;
 mo_success:
@@ -1144,7 +1200,7 @@ static int sema_msg_session_mo(SMSCenter *smsc, char* cbuff){
 	retresult = 0;  	/* deliver invoke */  
 	iret = sema_submit_result(smsc, rmsg, retresult); 
 	if(iret == -1) goto error;
-	deliver_invoke= (struct sm_deliver_invoke*) (*rmsg->msgbody);
+	deliver_invoke = (struct sm_deliver_invoke*) rmsg->msgbody;
 	if(deliver_invoke == NULL) goto msg_error;
 	sema_msglist_push(smsc->sema_mo, rmsg);  
 	return 1;
@@ -1153,7 +1209,7 @@ static int sema_msg_session_mo(SMSCenter *smsc, char* cbuff){
 	retresult = 0; 		/* let msg through */
 	sema_submit_result(smsc, rmsg, retresult);
 	if(iret == -1) goto error;
-	report_invoke = (struct sm_statusreport_invoke*)(*rmsg->msgbody);
+	report_invoke = (struct sm_statusreport_invoke*) rmsg->msgbody;
 	if(report_invoke != NULL)  
 	    gw_free(report_invoke);
     }
@@ -1167,7 +1223,7 @@ msg_error:
     error(0,"sema_mo session: Msg decode failed");
     return 0;
 error:  
-    error(0,"sema_mo session: device file error or memory allocated problem!");
+    error(0,"sema_mo session: device file error or memory allocation problem!");
     return -1;
 }
 
@@ -1191,7 +1247,7 @@ static int sema_decode_msg(sema_msg **desmsg, char* octsrc) {
     if(cmsgtype != 's' 		/* invoke reseult */
        && cmsgtype != 'M'  	/* deliver invoke */
        && cmsgtype != 'T'){  	/* report invoke */
-	info(0,"sema_decode: msg type not support");
+	info(0,"sema_decode: msg type not supported");
 	goto error_msg;
     }
 
@@ -1201,7 +1257,7 @@ static int sema_decode_msg(sema_msg **desmsg, char* octsrc) {
 				 &imsgencodetype,&imsgtopseg, &imsgfollownum);
 
     if(iret == -1){
-	info(0,"sema_decode: msg continue bit can not be interpret");
+	info(0,"sema_decode: msg continue bit can not be interpreted");
 	goto error_msg;
     }
 
@@ -1247,7 +1303,7 @@ static int sema_decode_msg(sema_msg **desmsg, char* octsrc) {
 	    if(iusedbyte < 1) goto error_submit;	
 	    memcpy(submit_result->accepttime, tmp, 4);
 	}
-	(*desmsg)->msgbody = (void**)&submit_result;
+	(*desmsg)->msgbody = submit_result;
 	break;
     case 'M': 
 	/* deliver invoke*/
@@ -1286,7 +1342,7 @@ static int sema_decode_msg(sema_msg **desmsg, char* octsrc) {
 	octetlen = 1;
 	receive_sm->DCS = get_variable_value(tmp, &octetlen);
 	if(receive_sm->DCS != ENCODE_IA5 && receive_sm->DCS !=ENCODE_GSM){
-	    info(0, "sema_decode, Data encoding scheme not support");
+	    info(0, "sema_decode, Data encoding scheme not supported");
 	    goto error_deliver;
 	}
 	/* protocol */ 
@@ -1348,7 +1404,7 @@ static int sema_decode_msg(sema_msg **desmsg, char* octsrc) {
 	iusedbyte = line_scan_IA5_hex(octsrc,14,tmp);
 	if(iusedbyte < 1) goto error_deliver;
 	memcpy(receive_sm->invoketime, tmp,14);
-	(*desmsg)->msgbody = (void**)&receive_sm;
+	(*desmsg)->msgbody = receive_sm;
 	break;
     case 'T': 
 	/* status report invoke */
@@ -1430,13 +1486,13 @@ static int sema_decode_msg(sema_msg **desmsg, char* octsrc) {
 	    goto error_receive;
 	}
 	memcpy(receive_report->invoketime ,tmp, 14);
-	(*desmsg)->msgbody = (void**)&receive_report;
+	(*desmsg)->msgbody = receive_report;
 	break;
     }
     return 1;
 
 no_msg:
-    info(0,"sema_decode: msg in empty");
+    info(0,"sema_decode: msg is empty");
     return 0;
 error_receive:
     gw_free(receive_report);
@@ -1462,7 +1518,7 @@ static int sema_encode_msg(sema_msg* pmsg, char* str) {
     switch(pmsg->type)
     {
     case 'S':
-	submit_sm = (struct sm_submit_invoke *)(*(pmsg->msgbody)); 
+	submit_sm = (struct sm_submit_invoke *) pmsg->msgbody; 
 	write_variable_value(submit_sm->msisdnlen, oc1byte); /*msisdn len*/
 	line_append_hex_IA5(IA5msg, oc1byte,1);
 	line_append_hex_IA5(IA5msg, 
