@@ -1,10 +1,66 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * Implementation of a SM/ASI SMSC module.
  *
  * Stipe Tolj <tolj@wapme-systems.de>
  *
  * This module connects to a CriticalPath InVoke SMS Center which
- * uses the SM/ASI protocoll. 
+ * uses the SM/ASI protocol. 
  * The module is heavily based on the SMPP module design.
  *
  * TODO:
@@ -50,7 +106,6 @@ static void dump_pdu(const char *msg, Octstr *id, SMASI_PDU *pdu)
 /************************************************************************/
 
 #define SMASI_DEFAULT_PORT          21500
-#define SMASI_RECONNECT_DELAY       10.0
 #define SMASI_DEFAULT_PRIORITY      0
 #define MAX_PENDING_SUBMITS         10
 #define SMASI_THROTTLING_SLEEP_TIME 15
@@ -93,7 +148,6 @@ typedef struct {
     long source_addr_npi;
     long dest_addr_ton;
     long dest_addr_npi;
-    long reconnect_delay;
     long priority;
     time_t throttling_err_time;
     int quitting;
@@ -123,7 +177,6 @@ static SMASI *smasi_create(SMSCConn *conn)
     smasi->dest_addr_npi = -1;
     smasi->my_number = NULL;
     smasi->port = 21500;
-    smasi->reconnect_delay = 10;
     smasi->quitting = 0;
     smasi->logged_off = 0;
     smasi->priority = 0;
@@ -225,7 +278,7 @@ static void unescape_data(Octstr *data)
             char byte = 0;
             int msb = octstr_get_char(data, pos + 1);
             int lsb = octstr_get_char(data, pos + 2);
-
+ 
             if (msb == '0') msb = 0;
             else if (msb >= '1' && msb <= '9') msb -= '1' + 1;
             else msb -= 'a' + 10;
@@ -281,44 +334,40 @@ static void encode_binary_data(Octstr *data)
 
 
 /*
- * Replaces a SMASI conform ASCII representation of binary data with the
- * original binary data octet string. Will abort data decoding if the ASCII
- * representation is invalid.
+ * Re-escape SMASI ASCII representation of binary data with the
+ * original binary data octet string.
+ * XXX this may be done by the internal parser routines too.
  */
 static void decode_binary_data(Octstr *data) 
 {
     long pos = 0;
-    Octstr * result = octstr_create("");
 
-    for (pos = 0; pos < octstr_len(data); pos += 3) {
+    while (pos < octstr_len(data)) {
         int check = octstr_get_char(data, pos);
 
-        if (check != ':') {
-            warning(0, "Malformed binary encoded data.");
-            return;
-        } else {
-            int byte = 0;
+        if (check == ':') {
+            Octstr *byte;
             int msb = octstr_get_char(data, pos + 1);
             int lsb = octstr_get_char(data, pos + 2);
 
-            if (msb == '0') msb = 0;
-            else if (msb >= '1' && msb <= '9') msb = msb - 48;
-            else msb = msb - 'a' + 10;
+            if (msb != -1 && lsb != -1) {
+                byte = octstr_create("");
+                octstr_append_char(byte, msb);
+                octstr_append_char(byte, lsb);
 
-            if (lsb == '0') lsb = 0;
-            else if (lsb >= '1' && lsb <= '9') lsb = lsb - 48;
-            else lsb = lsb - 'a' + 10;
+                if (octstr_hex_to_binary(byte) != -1) {
+                    /* Do inplace unescaping. */
+                    octstr_delete(data, pos, 3);
+                    octstr_insert(data, byte, pos);
+                } else {
+                    error(0, "Malformed binary encoded data.");
+                }
 
-            byte = msb << 4 | lsb;
-
-            octstr_append_char(result, byte);
+                octstr_destroy(byte);
+            }
         } 
+        pos++;
     } 
-
-    /* Replace ASCII representation with binary data octet string. */
-    octstr_delete(data, 0, octstr_len(data));
-    octstr_append(data, result);
-    octstr_destroy(result);
 }
 
 
@@ -328,12 +377,12 @@ static void decode_binary_data(Octstr *data)
 
 static Octstr *get_ton_npi_value(int override, int message) 
 {
-    if(override != -1) {
-        debug("bb.sms.smasi", 0, "SMASI: Manually forced source addr ton = %d", 
+    if (override != -1) {
+        debug("bb.sms.smasi", 0, "SMASI: Manually forced ton or npi to `%d'", 
               override);
-        return(octstr_format("%ld", override));
+        return (octstr_format("%ld", override));
     } else {
-        return(octstr_format("%ld", message));
+        return (octstr_format("%ld", message));
     }
 }
 
@@ -432,7 +481,7 @@ static SMASI_PDU *msg_to_pdu(SMASI *smasi, Msg *msg)
 
     /* If its a international number starting with +, lets remove the +. */
     if (octstr_get_char(pdu->u.SubmitReq.Destination, 0) == '+')
-        octstr_delete(pdu->u.SubmitReq.Destination, 0,1);
+        octstr_delete(pdu->u.SubmitReq.Destination, 0, 1);
 
     /* Do ton and npi override - if configured. Use values from message
      * otherwise.
@@ -450,23 +499,24 @@ static SMASI_PDU *msg_to_pdu(SMASI *smasi, Msg *msg)
     }
 
     /* Set encoding. */
-    if (msg->sms.coding != 0) {
-        if (msg->sms.coding == 1)
+    if (msg->sms.coding != DC_UNDEF) {
+        if (msg->sms.coding == DC_7BIT)
             pdu->u.SubmitReq.MsEncoding = octstr_create("7bit");
-        else if (msg->sms.coding == 2)
+        else if (msg->sms.coding == DC_8BIT)
             pdu->u.SubmitReq.MsEncoding = octstr_create("8bit");
-        else if (msg->sms.coding == 2)
+        else if (msg->sms.coding == DC_UCS2)
             pdu->u.SubmitReq.MsEncoding = octstr_create("16bit");
 
         /* Everything else will default to 7bit. */
     }
 
     /* Set messaging class - if within defined parameter range. */
-    if (msg->sms.mclass >= 0 && msg->sms.mclass <= 4)
-        pdu->u.SubmitReq.Class = octstr_format("%ld", (msg->sms.mclass - 1));
+    if (msg->sms.mclass != MC_UNDEF)
+        pdu->u.SubmitReq.Class = octstr_format("%ld", msg->sms.mclass);
 
     /* Set Protocol ID. */
-    pdu->u.SubmitReq.ProtocolID = octstr_format("%ld", msg->sms.pid);
+    pdu->u.SubmitReq.ProtocolID = octstr_format("%ld", 
+	(msg->sms.pid == SMS_PARAM_UNDEFINED ? 0 : msg->sms.pid));
 
     /* Check if SMS is binary. */
     if (msg->sms.udhdata && octstr_len(msg->sms.udhdata) > 0) {
@@ -508,65 +558,64 @@ static Msg *pdu_to_msg(SMASI_PDU *pdu)
     gw_assert(pdu->u.DeliverReq.Destination);
     gw_assert(pdu->u.DeliverReq.Body);
 
-    msg = msg_create(sms);;
+    msg = msg_create(sms);
 
     msg->sms.sender = octstr_duplicate(pdu->u.DeliverReq.Originator);
     msg->sms.receiver = octstr_duplicate(pdu->u.DeliverReq.Destination);
     msg->sms.msgdata = octstr_duplicate(pdu->u.DeliverReq.Body);
  
-    /* Unescape (non-binary) or decode (binary) data. */
-    if (pdu->u.DeliverReq.UserDataHeader &&
-        octstr_len(pdu->u.DeliverReq.UserDataHeader) > 0) {
-
-        msg->sms.udhdata = octstr_duplicate(pdu->u.DeliverReq.UserDataHeader);
-
-        decode_binary_data(msg->sms.msgdata);
-        decode_binary_data(msg->sms.udhdata);
-    } else {
-        unescape_data(msg->sms.msgdata);
-    } 
-
     /* Read priority. */
     if (pdu->u.DeliverReq.ProtocolId)
         if (octstr_parse_long(&msg->sms.pid, 
                               pdu->u.DeliverReq.ProtocolId, 0, 10) == -1)
-            msg->sms.pid = 0;
+            msg->sms.pid = SMS_PARAM_UNDEFINED;
 
     /* Read Coding. */
-    if (pdu->u.SubmitReq.MsEncoding) {
+    if (pdu->u.DeliverReq.MsEncoding) {
 
         /* Use specified coding. */
-        if (octstr_str_compare(pdu->u.SubmitReq.MsEncoding, "7bit") == 0)
-            msg->sms.coding = 1;
-        else if (octstr_str_compare(pdu->u.SubmitReq.MsEncoding, "8bit") == 0)
-            msg->sms.coding = 2;
-        else if (octstr_str_compare(pdu->u.SubmitReq.MsEncoding, "UCS2") == 0)
-            msg->sms.coding = 3;
-        else if (octstr_str_compare(pdu->u.SubmitReq.MsEncoding, "transparent") == 0)
-            msg->sms.coding = 2;
+        if (octstr_str_compare(pdu->u.DeliverReq.MsEncoding, "7bit") == 0)
+            msg->sms.coding = DC_7BIT;
+        else if (octstr_str_compare(pdu->u.DeliverReq.MsEncoding, "8bit") == 0)
+            msg->sms.coding = DC_8BIT;
+        else if (octstr_str_compare(pdu->u.DeliverReq.MsEncoding, "UCS2") == 0)
+            msg->sms.coding = DC_UCS2;
+        else if (octstr_str_compare(pdu->u.DeliverReq.MsEncoding, "transparent") == 0)
+            msg->sms.coding = DC_8BIT;
     } else {
 
         /* Determine specified coding according to udhdata presence. */
-        if (pdu->u.SubmitReq.UserDataHeader)
-            msg->sms.coding = 2;
+        if (pdu->u.DeliverReq.UserDataHeader)
+            msg->sms.coding = DC_8BIT;
         else
-            msg->sms.coding = 1;
+            msg->sms.coding = DC_7BIT;
     }
 
+    /* Unescape (non-binary) or decode (binary) data. */
+    if (msg->sms.coding == DC_8BIT) {
+
+        decode_binary_data(msg->sms.msgdata);
+        if (pdu->u.DeliverReq.UserDataHeader &&
+            octstr_len(pdu->u.DeliverReq.UserDataHeader) > 0) {
+            msg->sms.udhdata = octstr_duplicate(pdu->u.DeliverReq.UserDataHeader);
+            decode_binary_data(msg->sms.udhdata);
+        }
+
+    } else {
+        unescape_data(msg->sms.msgdata);
+    } 
+
     /* Read message class. */
-    if (pdu->u.SubmitReq.Class) {
-        if (octstr_parse_long(&msg->sms.mclass, 
-                              pdu->u.SubmitReq.Class, 0, 10) == -1)
-            msg->sms.mclass = 0;    /* Set to unspecified. */
-        else
-            msg->sms.mclass++;      /* Correct value mapping. */
-    }
+    if (pdu->u.DeliverReq.Class &&
+        octstr_parse_long(&msg->sms.mclass, 
+                          pdu->u.DeliverReq.Class, 0, 10) == -1)
+            msg->sms.mclass = MC_UNDEF;    /* Set to unspecified. */
 
     /* Read protocol ID. */
-    if (pdu->u.SubmitReq.ProtocolID)
-        if (octstr_parse_long(&msg->sms.pid, 
-                              pdu->u.SubmitReq.ProtocolID, 0, 10) == -1)
-            msg->sms.pid = 0;
+    if (pdu->u.DeliverReq.ProtocolId &&
+        octstr_parse_long(&msg->sms.pid, 
+                          pdu->u.DeliverReq.ProtocolId, 0, 10) == -1)
+            msg->sms.pid = SMS_PARAM_UNDEFINED;
 
     return msg;
 }
@@ -639,7 +688,7 @@ static int read_pdu(SMASI *smasi, Connection *conn, SMASI_PDU **pdu)
     
     os = smasi_pdu_read(conn);
     if (os == NULL) {
-        if (conn_eof(conn) || conn_read_error(conn)) 
+        if (conn_eof(conn) || conn_error(conn)) 
             return -1;
         return 0;
     }
@@ -671,6 +720,8 @@ static void handle_pdu(SMASI *smasi, Connection *conn,
         case DeliverReq:
             msg = pdu_to_msg(pdu);
 
+            msg_dump(msg, 0);
+
             if (smasi->my_number && octstr_len(smasi->my_number)) {
                 octstr_destroy(msg->sms.receiver);
                 msg->sms.receiver = octstr_duplicate(smasi->my_number);
@@ -708,7 +759,7 @@ static void handle_pdu(SMASI *smasi, Connection *conn,
                        octstr_get_cstr(pdu->u.SubmitConf.Sequence),
                        octstr_get_cstr(pdu->u.SubmitConf.MsgReference));
 
-                bb_smscconn_sent(smasi->conn, msg);
+                bb_smscconn_sent(smasi->conn, msg, NULL);
 
                 --(*pending_submits);
             }
@@ -732,7 +783,8 @@ static void handle_pdu(SMASI *smasi, Connection *conn,
                        octstr_get_cstr(smasi->conn->id));
             } else {
                 reason = SMSCCONN_FAILED_REJECTED;
-                bb_smscconn_send_failed(smasi->conn, msg, reason);
+                bb_smscconn_send_failed(smasi->conn, msg, reason,
+		            octstr_create("REJECTED"));
                 --(*pending_submits);
             }
             break;
@@ -790,7 +842,7 @@ static void handle_pdu(SMASI *smasi, Connection *conn,
  */
 static Connection *open_connection(SMASI *smasi) 
 {
-    Connection *conn = conn_open_tcp(smasi->host, smasi->port, NULL);
+    Connection *conn = conn_open_tcp_with_port(smasi->host, smasi->port, 0, smasi->conn->our_host);
 
     if (conn == NULL) {
         error(0, "SMASI[%s]: Couldn't connect to server.",
@@ -816,12 +868,18 @@ static Connection *open_connection(SMASI *smasi)
 static void send_messages(SMASI *smasi, Connection *conn, 
                           long *pending_submits) 
 {
+    double delay = 0;
+
     if (*pending_submits == -1) return;
 
+    if (smasi->conn->throughput) {
+        delay = 1.0 / smasi->conn->throughput;
+    }
+
     while (*pending_submits < MAX_PENDING_SUBMITS) {
         SMASI_PDU *pdu = NULL;
         /* Get next message, quit if none to be sent. */
-        Msg * msg = list_extract_first(smasi->msgs_to_send);
+        Msg *msg = list_extract_first(smasi->msgs_to_send);
 
         if (msg == NULL) break;
 
@@ -835,6 +893,10 @@ static void send_messages(SMASI *smasi, Connection *conn,
 
         smasi_pdu_destroy(pdu);
 
+        /* obey throughput speed limit, if any */
+        if (smasi->conn->throughput)
+            gwthread_sleep(delay);
+
         ++(*pending_submits);
     }
 }
@@ -860,15 +922,18 @@ static void smasi_thread(void *arg)
 
     smasi = arg;
 
+    /* Make sure we log into our own log-file if defined */
+    log_thread_to(smasi->conn->log_idx);
+    
     while (!smasi->quitting) {
 
         conn = open_connection(smasi);
         if (conn == NULL) {
             error(0, "SMASI[%s]: Could not connect to SMSC center " \
                   "(retrying in %ld seconds).",
-                  octstr_get_cstr(smasi->conn->id), smasi->reconnect_delay);
+                  octstr_get_cstr(smasi->conn->id), smasi->conn->reconnect_delay);
 
-            gwthread_sleep(smasi->reconnect_delay);
+            gwthread_sleep(smasi->conn->reconnect_delay);
             smasi->conn->status = SMSCCONN_RECONNECTING;
             continue;
         }
@@ -881,6 +946,13 @@ static void smasi_thread(void *arg)
             timeout = last_enquire_sent + smasi->enquire_link_interval
                         - date_universal_now(); 
 
+            /* wait for activity */
+            if (conn_wait(conn, timeout) == -1) {
+                error(0, "SMASI[%s]: I/O error or other error. Re-connecting.",
+                      octstr_get_cstr(smasi->conn->id));
+                break;
+            }
+
             /* Send logoff request if module is shutting down. */
             if (smasi->quitting && !logoff_already_sent) {
                 send_logoff(smasi, conn);
@@ -912,7 +984,7 @@ static void smasi_thread(void *arg)
             } 
 
             /* Check if connection broken. */
-            if (ret == -1 || conn_wait(conn, -1) == -1) {
+            if (ret == -1) {
                 error(0, "SMASI[%s]: I/O error or other error. Re-connecting.",
                       octstr_get_cstr(smasi->conn->id));
                 break;
@@ -1023,9 +1095,6 @@ static int init_configuration(SMASI *smasi, CfgGroup *config)
     smasi->my_number = cfg_get(config, octstr_imm("my-number"));
     if (cfg_get_integer(&smasi->port, config, octstr_imm("port")) == -1)
         smasi->port = SMASI_DEFAULT_PORT;
-    if (cfg_get_integer(&smasi->reconnect_delay, config,
-      octstr_imm("reconnect-delay")) == -1)
-        smasi->reconnect_delay = SMASI_RECONNECT_DELAY;
     if (cfg_get_integer(&smasi->source_addr_ton, config,
       octstr_imm("source-addr-ton")) == -1)
         smasi->source_addr_ton = SMASI_OVERRIDE_SOURCE_TON;
@@ -1034,10 +1103,10 @@ static int init_configuration(SMASI *smasi, CfgGroup *config)
         smasi->source_addr_npi = SMASI_OVERRIDE_SOURCE_NPI;
     if (cfg_get_integer(&smasi->dest_addr_ton, config,
       octstr_imm("dest-addr-ton")) == -1)
-        smasi->source_addr_ton = SMASI_OVERRIDE_DEST_TON;
+        smasi->dest_addr_ton = SMASI_OVERRIDE_DEST_TON;
     if (cfg_get_integer(&smasi->dest_addr_npi, config,
       octstr_imm("dest-addr-npi")) == -1)
-        smasi->source_addr_npi = SMASI_OVERRIDE_DEST_NPI;
+        smasi->dest_addr_npi = SMASI_OVERRIDE_DEST_NPI;
     if (cfg_get_integer(&smasi->priority, config,
       octstr_imm("priority")) == -1)
         smasi->priority = SMASI_DEFAULT_PRIORITY;
