@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * Wap_push_ota.c: implementation of push related requests of OTA protocol
  *
@@ -215,6 +271,25 @@ static void make_session_request(WAPEvent *e)
     wsp_event = wap_event_create(S_Unit_Push_Req);
     wsp_event->u.S_Unit_Push_Req.address_type =
         e->u.Pom_SessionRequest_Req.address_type;
+    if (e->u.Pom_SessionRequest_Req.smsc_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsc_id =
+            octstr_duplicate(e->u.Pom_SessionRequest_Req.smsc_id);
+    else
+        wsp_event->u.S_Unit_Push_Req.smsc_id = NULL;
+    if (e->u.Pom_SessionRequest_Req.dlr_url != NULL)
+        wsp_event->u.S_Unit_Push_Req.dlr_url =
+            octstr_duplicate(e->u.Pom_SessionRequest_Req.dlr_url);
+    else
+        wsp_event->u.S_Unit_Push_Req.dlr_url = NULL;
+    wsp_event->u.S_Unit_Push_Req.dlr_mask = e->u.Pom_SessionRequest_Req.dlr_mask;
+    if (e->u.Pom_SessionRequest_Req.smsbox_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsbox_id =
+            octstr_duplicate(e->u.Pom_SessionRequest_Req.smsbox_id);
+    else
+        wsp_event->u.S_Unit_Push_Req.smsbox_id = NULL;    
+    wsp_event->u.S_Unit_Push_Req.service_name = 
+        octstr_duplicate(e->u.Pom_SessionRequest_Req.service_name);
+
     wsp_event->u.S_Unit_Push_Req.push_id = 
         e->u.Pom_SessionRequest_Req.push_id;
     wsp_event->u.S_Unit_Push_Req.addr_tuple = 
@@ -281,8 +356,21 @@ static void make_unit_push_request(WAPEvent *e)
 {
     WAPEvent *wsp_event;
     List *push_headers;
+    Octstr *smsc_id;
+    Octstr *dlr_url;
+    Octstr *smsbox_id;
+    Octstr *push_body;
+    Octstr *service_name;
 
     gw_assert(e->type == Po_Unit_Push_Req);
+    gw_assert(e->u.Po_Unit_Push_Req.addr_tuple);
+    gw_assert(e->u.Po_Unit_Push_Req.service_name);
+
+    smsc_id = octstr_duplicate(e->u.Po_Unit_Push_Req.smsc_id); 
+    dlr_url = octstr_duplicate(e->u.Po_Unit_Push_Req.dlr_url);
+    smsbox_id = octstr_duplicate(e->u.Po_Unit_Push_Req.smsbox_id);
+    push_body = octstr_duplicate(e->u.Po_Unit_Push_Req.push_body);
+    service_name = octstr_duplicate(e->u.Po_Unit_Push_Req.service_name); 
     push_headers = add_push_flag(e);
 
     wsp_event = wap_event_create(S_Unit_Push_Req);
@@ -293,10 +381,24 @@ static void make_unit_push_request(WAPEvent *e)
 
     wsp_event->u.S_Unit_Push_Req.address_type = 
         e->u.Po_Unit_Push_Req.address_type;
-
-    if (e->u.Po_Unit_Push_Req.push_body != NULL)
-        wsp_event->u.S_Unit_Push_Req.push_body =
-	    octstr_duplicate(e->u.Po_Unit_Push_Req.push_body);
+    if (smsc_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsc_id = smsc_id;
+    else
+        wsp_event->u.S_Unit_Push_Req.smsc_id = NULL;  
+    if (dlr_url != NULL)      
+        wsp_event->u.S_Unit_Push_Req.dlr_url = dlr_url;        
+    else  
+        wsp_event->u.S_Unit_Push_Req.dlr_url = NULL;      
+    wsp_event->u.S_Unit_Push_Req.dlr_mask = e->u.Po_Unit_Push_Req.dlr_mask;            
+    if (smsbox_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsbox_id = smsbox_id;
+    else
+        wsp_event->u.S_Unit_Push_Req.smsbox_id = NULL;
+    wsp_event->u.S_Unit_Push_Req.service_name = service_name;
+    if (push_body != NULL)
+        wsp_event->u.S_Unit_Push_Req.push_body = push_body;
+    else
+        wsp_event->u.S_Unit_Push_Req.push_body = NULL;
 
     dispatch_to_wsp_unit(wsp_event);
     debug("wap.push.ota", 0, "OTA: made connectionless session service"
@@ -322,7 +424,8 @@ static void abort_push(WAPEvent *e)
 }
 
 /*
- * Add push flag into push headers. Push flag is defined in otaa, p. 17-18.
+ * Add push flag into push headers. Push flag is defined in ota, p. 17-18.
+ * If there is no flags set, no Push-Flag header is added.
  */
 static List *add_push_flag(WAPEvent *e)
 {
@@ -367,9 +470,11 @@ static List *add_push_flag(WAPEvent *e)
     push_flag = 0;
     push_flag = push_flag | authenticated | trusted | last;
     
+    if (push_flag) {
     buf = octstr_format("%d", push_flag);
     http_header_add(headers, "Push-Flag", octstr_get_cstr(buf)); 
     octstr_destroy(buf);
+    }
 
     return headers;
 }
@@ -542,7 +647,6 @@ static void bearerbox_address_destroy(BearerboxAddress *ba)
     if (ba == NULL)
         return;
 
-    mutex_lock(ba->mutex);
     octstr_destroy(ba->address);
     mutex_destroy(ba->mutex);
     gw_free(ba);
