@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * wap_push_pap_compiler.c - implementation of wap_push_pap_compiler.h inter-
  * face (compiling pap documents to Kannel WAPEvents)
@@ -196,6 +252,7 @@ static int set_attribute_value(Octstr *element_name, Octstr *attr_value,
                                Octstr *attr_name, WAPEvent **e);
 static int return_flag(Octstr *ros);
 static void wap_event_accept_or_create(Octstr *element_name, WAPEvent **e);
+static int parse_pap_value(Octstr *attr_name, Octstr *attr_value, WAPEvent **e);
 static int parse_push_message_value(Octstr *attr_name, Octstr *attr_value,
                                      WAPEvent **e);
 static int parse_address_value(Octstr *attr_name, Octstr *attr_value, 
@@ -218,7 +275,6 @@ static int parse_requirement(Octstr *attr_value);
 static int parse_priority(Octstr *attr_value);
 static int parse_delivery_method(Octstr *attr_value);
 static int parse_state(Octstr *attr_value);
-static int parse_address(Octstr **attr_value, long *type_of_address);
 static long parse_wappush_client_address(Octstr **address, long pos,
                                          long *type_of_address);
 static long parse_ppg_specifier(Octstr **address, long pos);
@@ -756,8 +812,9 @@ static int parse_element(xmlNodePtr node, WAPEvent **e, long *type_of_address,
     }
 
     if (i == NUM_ELEMENTS) {
+        debug("wap.push.pap.compiler", 0, "PAP COMPILER: unknown element:");
+        octstr_dump(name, 0);
         octstr_destroy(name);
-        debug("wap.push.pap.compiler", 0, "PAP COMPILER: unknown element");
         return -2;
     }
 
@@ -804,6 +861,7 @@ static int parse_attribute(Octstr *element_name, xmlAttrPtr attribute,
 
     nameos = octstr_imm("erroneous");
     attr_name = octstr_create(attribute->name);
+
     if (attribute->children != NULL)
         value = create_octstr_from_node(attribute->children);
     else
@@ -818,7 +876,9 @@ static int parse_attribute(Octstr *element_name, xmlAttrPtr attribute,
     }
 
     if (i == NUM_ATTRIBUTES) {
-        debug("wap.push.pap.compiler", 0, "PAP COMPILER: unknown attribute");
+        debug("wap.push.pap.compiler", 0, "PAP COMPILER: unknown attribute `%s' "
+              "within XML entity `%s'", octstr_get_cstr(attr_name), 
+              octstr_get_cstr(element_name));
         goto error;
     }
 
@@ -829,6 +889,7 @@ static int parse_attribute(Octstr *element_name, xmlAttrPtr attribute,
     if (pap_attributes[i].value == NULL) {
         ret = parse_attr_value(element_name, attr_name, value, e, 
                                type_of_address, is_any);
+
 	if (ret == -2) {
 	    goto error;
         } else {
@@ -844,8 +905,10 @@ static int parse_attribute(Octstr *element_name, xmlAttrPtr attribute,
     }
 
     if (octstr_compare(attr_name, nameos) != 0) {
-        debug("wap.push.pap.compiler", 0, "PAP COMPILER: unknown attribute"
-              " value");
+        debug("wap.push.pap.compiler", 0, "PAP COMPILER: unknown attribute "
+              "value `%s' for attribute `%s' within XML entity `%s'", 
+              octstr_get_cstr(value), octstr_get_cstr(attr_name), 
+              octstr_get_cstr(element_name));
         goto error;
     }
 
@@ -872,6 +935,36 @@ parsed:
     return ret;
 }
 
+/* 
+ * Attribute value parsing functions for the PAP element.
+ * Defined in PAP, chapter 8.1.  
+ */
+static int parse_pap_value(Octstr *attr_name, Octstr *attr_value, WAPEvent **e)
+{
+    Octstr *ros;
+
+    if (*e != NULL)
+        wap_event_dump(*e);
+
+    ros = octstr_imm("erroneous");
+    if (octstr_compare(attr_name, octstr_imm("product-name")) == 0) {
+        /* 
+         * XXX This is a kludge. 
+         * We can't add the product-name value to the WAPEvent, because
+         * the wap_event_create() is created in the deeper layer, which
+         * means as soon as we see <push-message> or <reponse-message>.
+         * But we would have to decide which WAPEvent to create while 
+         * being on the higher <pap> level. 
+         * How's this to be solved?! -- Stipe
+         */
+        return 0;
+    } 
+
+    debug("wap.push.pap.compiler", 0, "PAP COMPILER: unknown pap"
+          " element attribute `%s'", octstr_get_cstr(attr_name));
+    return -2;
+}  
+
 /* 
  * Value parsing functions return the newly created wap event containing 
  * attribute value from pap source, if successfull; NULL otherwise . Value 
@@ -918,7 +1011,7 @@ static int parse_push_message_value(Octstr *attr_name, Octstr *attr_value,
 
 /*
  * When there is no legal address to be stored in field (either parsing was
- * unsuccessfull or an unimplemented address format was requested by the push
+ * unsuccessful or an unimplemented address format was requested by the push
  * initiator) we use value "erroneous". This is necessary, because this a 
  * mandatory field.
  *
@@ -1161,7 +1254,9 @@ static int parse_attr_value(Octstr *element_name, Octstr *attr_name,
 
     wap_event_accept_or_create(element_name, e);
 
-    if (octstr_compare(element_name, octstr_imm("push-message")) == 0) {
+    if (octstr_compare(element_name, octstr_imm("pap")) == 0) {
+        return parse_pap_value(attr_name, attr_value, e);
+    } else if (octstr_compare(element_name, octstr_imm("push-message")) == 0) {
         return parse_push_message_value(attr_name, attr_value, e);
     } else if (octstr_compare(element_name, octstr_imm("address")) == 0) {
         return parse_address_value(attr_name, attr_value, e, type_of_address);
@@ -1393,7 +1488,7 @@ static int parse_state(Octstr *attr_value)
  *           -2, address parsing error  
  */
 
-static int parse_address(Octstr **address, long *type_of_address)
+int parse_address(Octstr **address, long *type_of_address)
 {
     long pos;
     Octstr *copy;
@@ -1405,8 +1500,14 @@ static int parse_address(Octstr **address, long *type_of_address)
     if (octstr_get_char(*address, 0) == '/')
         octstr_delete(*address, 0, 1);
 
-    if ((pos = parse_ppg_specifier(address, pos)) < 0) {
-        warning(0, "illegal ppg specifier");
+/*
+ * WAP-209, chapter 8 states that addresses with telephone numbers
+ * should not have a ppg specifier. WAP-151 grammar, however, makes it
+ * mandatory. Best way to solve this contradiction seems to be regarding
+ * ppg specifier optional - MMSC is very important type of pi.
+ */
+    if (octstr_search_char(*address, '@', 0) >= 0) {
+        if ((pos = parse_ppg_specifier(address, pos)) < 0)
         return -2;
     }
 
@@ -1442,7 +1543,8 @@ static long parse_wappush_client_address(Octstr **address, long pos,
 }
 
 /*
- * We are not interested of ppg specifier, but we must check its format.
+ * We are not interested of ppg specifier, but we must check its format,
+ * if we find it - it is optional.
  */
 static long parse_ppg_specifier(Octstr **address, long pos)
 {
@@ -1455,8 +1557,6 @@ static long parse_ppg_specifier(Octstr **address, long pos)
 	    octstr_delete(*address, pos, 1);
             --pos;
         } else {
-            debug("wap.push.pap.compiler", 0, "PAP COMPILER: missing"
-                  " separator .");
 	    return -2;
         }
 
@@ -1472,8 +1572,6 @@ static long parse_ppg_specifier(Octstr **address, long pos)
     }
 
     if (pos < 0) {
-       debug("wap.push.pap.compiler", 0, "PAP COMPILER: missing separator /"
-             " or @");
        return -2;
     }
 
@@ -1482,7 +1580,6 @@ static long parse_ppg_specifier(Octstr **address, long pos)
 
 /*
  * Output: the type of a client address.
- *
  * Return a negative value, when error, positive (the position of the parsing 
  * cursor) otherwise.
  */
@@ -1552,22 +1649,32 @@ parse_error:
     return -2;
 }
 
+/*
+ * XXX We have a kludge here. WAP-249-PPGService-20010713-a defines in 
+ * section 6.1 the constant strings "WAPPUSH" and "TYPE" in upper-case. 
+ * But in the examples of section 6.2 they use lower-case too. Some PI
+ * vendors (ie. Jatayuu's MMSC) have implemented lower-case in their PAP 
+ * documents. So we'll support this too for sake of operatibility -- st.
+ */
 static long parse_constant(const char *field_name, Octstr **address, long pos)
 {
-    size_t i,    
-           size;
+    size_t i, size;
     Octstr *nameos;
 
     nameos = octstr_format("%s", field_name);
     size = octstr_len(nameos);
     i = 0;
     
+    /* convert both to lower case, see above note */
+    octstr_convert_range(nameos, 0, octstr_len(nameos), tolower);
+    octstr_convert_range(*address, 0, octstr_len(*address), tolower);
+    
     while (octstr_get_char(*address, pos - i)  == 
                octstr_get_char(nameos, size-1 - i) && i <  size) {
         ++i;
     }
 
-    while (octstr_get_char(*address, pos) != 
+    while ((octstr_len(*address) > 0) && octstr_get_char(*address, pos) !=
                octstr_get_char(nameos, 0) && pos >= 0) {
         pos = drop_character(address, pos);
     }
