@@ -1,4 +1,60 @@
-/*
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
+/* 
  * wml_compiler.c - compiling WML to WML binary
  *
  * This is an implemention for WML compiler for compiling the WML text 
@@ -9,9 +65,6 @@
  * Tuomas Luttinen for Wapit Ltd.
  */
 
-
-#include "config.h"
-
 #include <time.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -34,6 +87,38 @@
  * Declarations of data types. 
  */
 
+struct wml_externalid_t {
+    char *string;
+    char value;
+};
+
+typedef struct wml_externalid_t wml_externalid_t;
+
+static wml_externalid_t wml_externalid[] = {
+    { "-//WAPFORUM//DTD WML 1.3//EN", 0x0A },
+    { "-//WAPFORUM//DTD WML 1.2//EN", 0x09 },
+    { "-//WAPFORUM//DTD WML 1.1//EN", 0x04 },
+    { "-//WAPFORUM//DTD WML 1.0//EN", 0x02 }
+};
+
+#define NUMBER_OF_WML_EXTERNALID sizeof(wml_externalid)/sizeof(wml_externalid[0])
+
+struct wbxml_version_t {
+    char *string;
+    char value;
+};
+
+typedef struct wbxml_version_t wbxml_version_t;
+
+static wbxml_version_t wbxml_version[] = {
+    { "1.3", 0x03 },
+    { "1.2", 0x02 },
+    { "1.1", 0x01 },
+};
+
+#define NUMBER_OF_WBXML_VERSION sizeof(wbxml_version)/sizeof(wbxml_version[0])
+
+
 typedef enum { NOESC, ESC, UNESC, FAILED } var_esc_t;
 
 
@@ -146,16 +231,16 @@ List *wml_URL_values_list;
  */
 
 static int parse_document(xmlDocPtr document, Octstr *charset, 
-			  wml_binary_t **wbxml);
+			  wml_binary_t **wbxml, Octstr *version);
 
 static int parse_node(xmlNodePtr node, wml_binary_t **wbxml);
 static int parse_element(xmlNodePtr node, wml_binary_t **wbxml);
 static int parse_attribute(xmlAttrPtr attr, wml_binary_t **wbxml);
-static int parse_attr_value(Octstr *attr_value, List *tokens, 
-			    wml_binary_t **wbxml);
+static int parse_attr_value(Octstr *attr_value, List *tokens,
+			    wml_binary_t **wbxml, int charset, var_esc_t default_esc);
 static int parse_text(xmlNodePtr node, wml_binary_t **wbxml);
 static int parse_cdata(xmlNodePtr node, wml_binary_t **wbxml);
-static int parse_st_octet_string(Octstr *ostr, int cdata, wml_binary_t **wbxml);
+static int parse_st_octet_string(Octstr *ostr, int cdata, var_esc_t default_esc, wml_binary_t **wbxml);
 static void parse_st_end(wml_binary_t **wbxml);
 static void parse_entities(Octstr *wml_source);
 
@@ -163,10 +248,11 @@ static void parse_entities(Octstr *wml_source);
  * Variable functions. These functions are used to find and parse variables.
  */
 
-static int parse_variable(Octstr *text, int start, Octstr **output, 
+static int parse_variable(Octstr *text, int start, var_esc_t default_esc, Octstr **output, 
 			  wml_binary_t **wbxml);
 static Octstr *get_variable(Octstr *text, int start);
-static var_esc_t check_variable_syntax(Octstr *variable);
+static var_esc_t check_variable_syntax(Octstr *variable, var_esc_t default_esc);
+
 
 /*
  * wml_binary-functions. These are used to create, destroy and modify
@@ -232,6 +318,7 @@ static unsigned long string_table_add(Octstr *ostr, wml_binary_t **wbxml);
 static void string_table_apply(Octstr *ostr, wml_binary_t **wbxml);
 static void string_table_output(Octstr *ostr, wml_binary_t **wbxml);
 
+
 /***********************************************************************
  * Implementations of the functions declared in wml_compiler.h.
  */
@@ -240,62 +327,96 @@ static void string_table_output(Octstr *ostr, wml_binary_t **wbxml);
  * The actual compiler function. This operates as interface to the compiler.
  * For more information, look wml_compiler.h. 
  */
-
-int wml_compile(Octstr *wml_text,
-		Octstr *charset,
-		Octstr **wml_binary)
+int wml_compile(Octstr *wml_text, Octstr *charset, Octstr **wml_binary,
+                Octstr *version)
 {
     int ret = 0;
     size_t size;
     xmlDocPtr pDoc = NULL;
     char *wml_c_text;
     wml_binary_t *wbxml = NULL;
+    Octstr *encoding = NULL;
 
     *wml_binary = octstr_create("");
     wbxml = wml_binary_create();
 
     /* Remove the extra space from start and the end of the WML Document. */
-
     octstr_strip_blanks(wml_text);
 
     /* Check the WML-code for \0-characters and for WML entities. Fast patch.
        -- tuo */
-
     parse_entities(wml_text);
-    set_charset(wml_text, charset);
-	    
+
+    /* transcode from charset to UTF-8 */
+    if (charset && octstr_len(charset) && 
+        octstr_case_compare(charset, octstr_imm("UTF-8")) == -1) {
+        debug("wml_compile", 0, "WML compiler: Transcoding from <%s> to UTF-8", 
+              octstr_get_cstr(charset));
+        set_charset(wml_text, charset);
+    }
+
+    /* 
+     * If we did not set the character set encoding yet, then obviously
+     * there was no charset argument in the Content-Type HTTP reply header.
+     * We have to scan the xml preamble line for an explicite encoding
+     * definition to allow transcoding from UTF-8 to that charset after 
+     * libxml2 did all it's parsing magic. (Keep in mind libxml2 uses UTF-8
+     * as internal encoding.) -- Stipe
+     */
+
+    /* 
+     * We will trust the xml preamble encoding more then the HTTP header 
+     * charset definition.
+     */
+    if ((encoding = find_charset_encoding(wml_text)) != NULL) {
+        /* ok, we rely on the xml preamble encoding */
+    } else if (charset && octstr_len(charset) > 0) {
+        /* we had a HTTP response charset, use this */
+        encoding = octstr_duplicate(charset);
+    } else {
+        /* we had none, so use UTF-8 as default */
+        encoding = octstr_create("UTF-8");
+    }
+
     size = octstr_len(wml_text);
     wml_c_text = octstr_get_cstr(wml_text);
 
     if (octstr_search_char(wml_text, '\0', 0) != -1) {    
-	error(0, 
-	      "WML compiler: Compiling error: "
-	      "\\0 character found in the middle of the WML source.");
-	ret = -1;
+        error(0, "WML compiler: Compiling error: "
+                 "\\0 character found in the middle of the WML source.");
+        ret = -1;
     } else {
-	/* 
-	 * An empty octet string for the binary output is created, the wml 
-	 * source is parsed into a parsing tree and the tree is then compiled 
-	 * into binary.
-	 */
 
-	pDoc = xmlParseMemory(wml_c_text, size);
+        /* 
+         * An empty octet string for the binary output is created, the wml 
+         * source is parsed into a parsing tree and the tree is then compiled 
+         * into binary.
+         */
 
-	if(pDoc != NULL) {
-	    ret = parse_document(pDoc, charset, &wbxml);
-	    wml_binary_output(*wml_binary, wbxml);
-	} else {    
-	    error(0, 
-		  "WML compiler: Compiling error: "
-		  "libxml returned a NULL pointer");
-	    ret = -1;
-	}
+        pDoc = xmlParseMemory(wml_c_text, size);
+       
+        if (pDoc != NULL) {
+            /* 
+             * If we have a set internal encoding, then apply this information 
+             * to the XML parsing tree document for later transcoding ability.
+             */
+            if (encoding)
+                pDoc->charset = xmlParseCharEncoding(octstr_get_cstr(encoding));
+
+            ret = parse_document(pDoc, encoding, &wbxml, version);
+            wml_binary_output(*wml_binary, wbxml);
+        } else {    
+            error(0, "WML compiler: Compiling error: "
+                     "libxml returned a NULL pointer");
+            ret = -1;
+        }
     }
 
     wml_binary_destroy(wbxml);
+    octstr_destroy(encoding);
 
     if (pDoc) 
-        xmlFreeDoc (pDoc);
+        xmlFreeDoc(pDoc);
 
     return ret;
 }
@@ -442,27 +563,71 @@ static int parse_node(xmlNodePtr node, wml_binary_t **wbxml)
  * The function outputs the WBXML version, WML public id and the
  * character set values into start of the wbxml.
  */
-
-static int parse_document(xmlDocPtr document, Octstr *charset, 
-			  wml_binary_t **wbxml)
+static int parse_document(xmlDocPtr document, Octstr *charset,
+                          wml_binary_t **wbxml, Octstr *version)
 {
-    Octstr *chars;
     xmlNodePtr node;
+    Octstr *externalID = NULL;
+    long i;
 
     if (document == NULL) {
-	error(0, "WML compiler: XML parsing failed, no parsed document.");
-	error(0, "Most probably an error in the WML source.");
-	return -1;
+        error(0, "WML compiler: XML parsing failed, no parsed document.");
+        error(0, "Most probably an error in the WML source.");
+        return -1;
     }
 
-    /* XXX - A bad hack, WBXML version is assumed to be 1.1. -tuo */
-    (*wbxml)->wbxml_version = 0x01; /* WBXML Version number 1.1 */
-    (*wbxml)->wml_public_id = 0x04; /* WML 1.1 Public ID */
+    /* Return WBXML version dependent on device given Encoding-Version */
+    if (version == NULL) {
+        (*wbxml)->wbxml_version = 0x01; /* WBXML Version number 1.1 */
+        info(0, "WBXML: No wbxml version given, assuming 1.1");
+    } else {
+        for (i = 0; i < NUMBER_OF_WBXML_VERSION; i++) {
+            if (octstr_compare(version, octstr_imm(wbxml_version[i].string)) == 0) {
+                (*wbxml)->wbxml_version = wbxml_version[i].value;
+                debug("parse_document",0,"WBXML: Encoding with wbxml version <%s>",
+                      octstr_get_cstr(version));
+                break;
+            }
+        }
+        if (i == NUMBER_OF_WBXML_VERSION) {
+            (*wbxml)->wbxml_version = 0x01; /* WBXML Version number 1.1 */
+            warning(0, "WBXML: Unknown wbxml version, assuming 1.1 (<%s> is unknown)",
+                    octstr_get_cstr(version));
+        }
+    }
+
+    /* Return WML Version dependent on xml ExternalID string */
+    if ((document->intSubset != NULL) && (document->intSubset->ExternalID != NULL))    
+        externalID = octstr_create(document->intSubset->ExternalID);
+    if (externalID == NULL) {
+        (*wbxml)->wml_public_id = 0x04; /* WML 1.1 Public ID */
+        warning(0, "WBXML: WML without ExternalID, assuming 1.1");
+    } else {
+        for (i = 0; i < NUMBER_OF_WML_EXTERNALID; i++) {
+            if (octstr_compare(externalID, octstr_imm(wml_externalid[i].string)) == 0) {
+                (*wbxml)->wml_public_id = wml_externalid[i].value;
+                debug("parse_document",0,"WBXML: WML with ExternalID <%s>",
+                      octstr_get_cstr(externalID));
+                break;
+            }
+        }
+        if (i == NUMBER_OF_WML_EXTERNALID) {
+            (*wbxml)->wml_public_id = 0x04; /* WML 1.1 Public ID */
+            warning(0, "WBXML: WML with unknown ExternalID, assuming 1.1 "
+                    "(<%s> is unknown)",
+                    octstr_get_cstr(externalID));
+        }
+    }
+    octstr_destroy(externalID);
+    
     (*wbxml)->string_table_length = 0x00; /* String table length=0 */
 
-    chars = octstr_create("UTF-8");
-    (*wbxml)->character_set = parse_charset(chars);
-    octstr_destroy(chars);
+    /*
+     * Make sure we set the charset encoding right. If none is given
+     * then set UTF-8 as default.
+     */
+    (*wbxml)->character_set = charset ? 
+        parse_charset(charset) : parse_charset(octstr_imm("UTF-8"));
 
     node = xmlDocGetRootElement(document);
     string_table_build(node, wbxml);
@@ -529,7 +694,7 @@ static int parse_element(xmlNodePtr node, wml_binary_t **wbxml)
 		add_end_tag = 1;
 	}
 	output_st_char(wbxml_hex, wbxml);
-	output_st_char(string_table_add(octstr_duplicate(name), wbxml), wbxml);
+	octstr_append_uintvar((*wbxml)->wbxml_string,string_table_add(octstr_duplicate(name), wbxml));
 	warning(0, "WML compiler: Unknown tag in WML source: <%s>", 
 		octstr_get_cstr(name));
     }
@@ -581,8 +746,19 @@ static int parse_attribute(xmlAttrPtr attr, wml_binary_t **wbxml)
 	if (attr->children == NULL || 
 	    (hit = list_search(attribute->value_list, (void *)pattern, 
 			       hash_cmp)) == NULL) {
-		wbxml_hex = attribute->binary;
-		output_st_char(wbxml_hex, wbxml);
+                if(attribute->binary == 0x00) {
+                    warning(0, "WML compiler: can't compile attribute %s%s%s%s", 
+                               octstr_get_cstr(attribute->attribute), 
+			       (attr->children != NULL ? "=\"": ""), 
+			       (attr->children != NULL ? octstr_get_cstr(pattern) : ""), 
+			       (attr->children != NULL ? "\"": ""));
+	            wbxml_hex = WBXML_LITERAL;
+	            output_st_char(wbxml_hex, wbxml);
+	            output_st_char(string_table_add(octstr_duplicate(name), wbxml), wbxml);
+		} else {
+		    wbxml_hex = attribute->binary;
+		    output_st_char(wbxml_hex, wbxml);
+		}
 	} else if (hit->binary) {
 	    wbxml_hex = hit->binary;
 	    coded_length = octstr_len(hit->item);
@@ -594,12 +770,16 @@ static int parse_attribute(xmlAttrPtr attr, wml_binary_t **wbxml)
 	   string. */
 	wbxml_hex = WBXML_LITERAL;
 	output_st_char(wbxml_hex, wbxml);
-	output_st_char(string_table_add(octstr_duplicate(name), wbxml), wbxml);
+	octstr_append_uintvar((*wbxml)->wbxml_string,string_table_add(octstr_duplicate(name), wbxml));
 	warning(0, "WML compiler: Unknown attribute in WML source: <%s>", 
 		octstr_get_cstr(name));
     }
 
     if (status >= 0) {
+	var_esc_t default_esc;
+
+	default_esc = (octstr_str_compare (name, "href") == 0) ? ESC : NOESC;
+
 	/* The rest of the attribute is coded as a inline string. */
 	if (pattern != NULL && 
 	    coded_length < (int) octstr_len(pattern)) {
@@ -611,10 +791,10 @@ static int parse_attribute(xmlAttrPtr attr, wml_binary_t **wbxml)
 
 	    if (check_if_url(wbxml_hex))
 		status = parse_attr_value(p, wml_URL_values_list,
-					  wbxml);
+					  wbxml, attr->doc->charset, default_esc);
 	    else
 		status = parse_attr_value(p, wml_attr_values_list,
-					  wbxml);
+					  wbxml, attr->doc->charset, default_esc);
 	    if (status != 0)
 		error(0, 
 		      "WML compiler: could not output attribute "
@@ -639,11 +819,28 @@ static int parse_attribute(xmlAttrPtr attr, wml_binary_t **wbxml)
  */
 
 static int parse_attr_value(Octstr *attr_value, List *tokens,
-			    wml_binary_t **wbxml)
+			    wml_binary_t **wbxml, int charset, var_esc_t default_esc)
 {
     int i, pos, wbxml_hex;
     wml_hash_t *temp = NULL;
     Octstr *cut_text = NULL;
+    char *tmp;
+
+    /*
+     * Beware that libxml2 does internal encoding in UTF-8 while parsing.
+     * So if our original WML source had a different encoding set, we have
+     * to transcode at least here. Only transcode if target encoding differs
+     * from libxml2's internal encoding (UTF-8).
+     */
+    tmp = (char*) xmlGetCharEncodingName(charset);
+    if (charset != XML_CHAR_ENCODING_UTF8 && 
+        charset_convert(attr_value, "UTF-8", 
+                        tmp) != 0) {
+        error(0, "Failed to convert XML attribute value from charset "
+                 "<%s> to <%s>, will leave as is.", "UTF-8", 
+                 tmp ? tmp : "(undef)");
+    }
+
 
     /*
      * The attribute value is search for text strings that can be replaced 
@@ -657,7 +854,7 @@ static int parse_attr_value(Octstr *attr_value, List *tokens,
     /* A fast patch to allow reserved names to be variable names. May produce 
        a little longer binary at some points. --tuo */
     if (octstr_search_char(attr_value, '$', 0) >= 0) {
-	if (parse_st_octet_string(attr_value, 0, wbxml) != 0)
+	if (parse_st_octet_string(attr_value, 0, default_esc, wbxml) != 0)
 	    return -1;
     } else {
 
@@ -680,7 +877,7 @@ static int parse_attr_value(Octstr *attr_value, List *tokens,
 		gw_assert(pos <= octstr_len(attr_value));
 	
 		cut_text = octstr_copy(attr_value, 0, pos);
-		if (parse_st_octet_string(cut_text, 0, wbxml) != 0)
+		if (parse_st_octet_string(cut_text, 0, default_esc, wbxml) != 0)
 		    return -1;
 		octstr_destroy(cut_text);
 	    
@@ -699,9 +896,9 @@ static int parse_attr_value(Octstr *attr_value, List *tokens,
 
 	if ((int) octstr_len(attr_value) > 0) {
 	    if (i < list_len(tokens))
-		parse_attr_value(attr_value, tokens, wbxml);
+		parse_attr_value(attr_value, tokens, wbxml, charset, default_esc);
 	    else
-		if (parse_st_octet_string(attr_value, 0, wbxml) != 0)
+		if (parse_st_octet_string(attr_value, 0, default_esc, wbxml) != 0)
 		    return -1;
 	}
     }
@@ -731,8 +928,24 @@ static int parse_text(xmlNodePtr node, wml_binary_t **wbxml)
 {
     int ret;
     Octstr *temp;
+    char* tmp;
 
-    temp = create_octstr_from_node(node);
+    temp = create_octstr_from_node(node); /* returns string in UTF-8 */
+
+    /*
+     * Beware that libxml2 does internal encoding in UTF-8 while parsing.
+     * So if our original WML source had a different encoding set, we have
+     * to transcode at least here. Only transcode if target encoding differs
+     * from libxml2's internal encoding (UTF-8).
+     */
+    tmp = (char*) xmlGetCharEncodingName(node->doc->charset);
+    if (node->doc->charset != XML_CHAR_ENCODING_UTF8 && 
+        charset_convert(temp, "UTF-8", 
+                        tmp) != 0) {
+        error(0, "Failed to convert XML text entity from charset "
+                 "<%s> to <%s>, will leave as is.", "UTF-8", 
+                 tmp ? tmp : "(undef)");
+    }
 
     octstr_shrink_blanks(temp);
     if (!check_if_emphasis(node->prev) && !check_if_emphasis(node->next))
@@ -741,7 +954,7 @@ static int parse_text(xmlNodePtr node, wml_binary_t **wbxml)
     if (octstr_len(temp) == 0)
         ret = 0;
     else 
-        ret = parse_st_octet_string(temp, 0, wbxml);
+        ret = parse_st_octet_string(temp, 0, NOESC, wbxml);
 
     /* Memory cleanup. */
     octstr_destroy(temp);
@@ -764,7 +977,7 @@ static int parse_cdata(xmlNodePtr node, wml_binary_t **wbxml)
 
     temp = create_octstr_from_node(node);
 
-    parse_st_octet_string(temp, 1, wbxml);
+    parse_st_octet_string(temp, 1, NOESC, wbxml);
     
     /* Memory cleanup. */
     octstr_destroy(temp);
@@ -785,7 +998,7 @@ static int parse_cdata(xmlNodePtr node, wml_binary_t **wbxml)
  * Parsed variable is returned as an octet string in Octstr **output.
  */
 
-static int parse_variable(Octstr *text, int start, Octstr **output, 
+static int parse_variable(Octstr *text, int start, var_esc_t default_esc, Octstr **output, 
 			  wml_binary_t **wbxml)
 {
     var_esc_t esc;
@@ -808,7 +1021,7 @@ static int parse_variable(Octstr *text, int start, Octstr **output,
 	else
 	    ret = octstr_len(variable) + 1;
 
-	if ((esc = check_variable_syntax(variable)) != FAILED)
+	if ((esc = check_variable_syntax(variable, default_esc)) != FAILED)
 	    output_variable(variable, output, esc, wbxml);
 	else
 	    octstr_destroy(variable);
@@ -866,7 +1079,7 @@ static Octstr *get_variable(Octstr *text, int start)
  * escape mode it has. Octstr *variable contains the variable string.
  */
 
-static var_esc_t check_variable_syntax(Octstr *variable)
+static var_esc_t check_variable_syntax(Octstr *variable, var_esc_t default_esc)
 {
     Octstr *escape;
     char ch;
@@ -896,7 +1109,7 @@ static var_esc_t check_variable_syntax(Octstr *variable)
 	}
 	octstr_destroy(escape);
     } else
-	ret = NOESC;
+	ret = default_esc;
 
     ch = octstr_get_char(variable, 0);
     if (!(isalpha((int)ch)) && ch != '_') {
@@ -923,7 +1136,7 @@ static var_esc_t check_variable_syntax(Octstr *variable)
  * not. Returns 0 for success, -1 for error.
  */
 
-static int parse_st_octet_string(Octstr *ostr, int cdata, wml_binary_t **wbxml)
+static int parse_st_octet_string(Octstr *ostr, int cdata, var_esc_t default_esc, wml_binary_t **wbxml)
 {
     Octstr *output, *var, *temp = NULL;
     int var_len;
@@ -949,7 +1162,7 @@ static int parse_st_octet_string(Octstr *ostr, int cdata, wml_binary_t **wbxml)
 		octstr_destroy(temp);
 	    }
 	  
-	    if ((var_len = parse_variable(ostr, pos, &var, wbxml)) > 0)	{
+	    if ((var_len = parse_variable(ostr, pos, default_esc, &var, wbxml)) > 0)	{
 		if (octstr_len(var) > 0) {
 		    if (octstr_get_char(var, 0) == '$')
 			/*
@@ -1047,7 +1260,7 @@ static void parse_entities(Octstr *wml_source)
 
 /*
  * wml_binary_create - reserves memory for the wml_binary_t and sets the 
- * fields to zeroes and NULLs.
+ * fields to zeros and NULLs.
  */
 
 static wml_binary_t *wml_binary_create(void)
@@ -1355,7 +1568,7 @@ static var_esc_t check_variable_name(xmlNodePtr node)
 	return FAILED;
     }
 
-    ret = check_variable_syntax(name);
+    ret = check_variable_syntax(name, NOESC);
 
     octstr_destroy(name);
     return ret;
