@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * cfg.c - configuration file handling
  *
@@ -371,7 +427,11 @@ int cfg_read(Cfg *cfg)
                     /* check if included file is a directory */
                     lstat(octstr_get_cstr(filename), &filestat);
 
-                    /* is a directory. create a list with files */
+                    /* 
+                     * is a directory, create a list with files of
+                     * this directory and load all as part of the
+                     * whole configuration.
+                     */
                     if (S_ISDIR(filestat.st_mode)) {
                         DIR *dh;
                         struct dirent *diritem;
@@ -403,7 +463,7 @@ int cfg_read(Cfg *cfg)
                     }
 
                     /* include files */
-                    while((file = list_extract_first(files)) != NULL) {
+                    while ((file = list_extract_first(files)) != NULL) {
 
                         list_insert(stack, 0, octstr_duplicate(file)); 
                         debug("gwlib.cfg", 0, "Loading include file `%s' (on line %ld of file %s).",  
@@ -422,10 +482,10 @@ int cfg_read(Cfg *cfg)
                         } 
                  
                         list_destroy(expand, NULL); 
-                        cfgloc_destroy(loc_inc); 
-
+                        cfgloc_destroy(loc_inc);
+                        octstr_destroy(file);
                     }
-                list_destroy(files, octstr_destroy_item);
+                    list_destroy(files, octstr_destroy_item);
                 } 
                 octstr_destroy(filename); 
             }  
@@ -512,6 +572,10 @@ Octstr *cfg_get_real(CfgGroup *grp, Octstr *varname, const char *file,
 {
     Octstr *os;
 
+    if(grp == NULL) 
+    	panic(0, "Trying to fetch variable `%s' in non-existing group",
+	      octstr_get_cstr(varname));
+
     if (grp->name != NULL && !is_allowed_in_group(grp->name, varname))
     	panic(0, "Trying to fetch variable `%s' in group `%s', not allowed.",
 	      octstr_get_cstr(varname), octstr_get_cstr(grp->name));
@@ -648,3 +712,20 @@ void cfg_dump(Cfg *cfg)
 
     debug("gwlib.cfg", 0, "Dump ends.");
 }
+
+void cfg_dump_all(void)
+{
+    #define OCTSTR(name) \
+        printf("%s = <please consult user doc>\n", #name);
+    #define SINGLE_GROUP(name, fields) \
+        printf("#\n#  Single Group\n#\n"); \
+        printf("group = %s\n", #name); \
+        fields; \
+        printf("\n\n");
+    #define MULTI_GROUP(name, fields) \
+        printf("#\n#  Multi Group\n#\n"); \
+        printf("group = %s\n", #name); \
+        fields; \
+        printf("\n\n");
+    #include "cfg.def"
+}
