@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * gwthread-pthread.c - implementation of gwthread.h using POSIX threads.
  *
@@ -33,6 +89,7 @@ struct threadinfo
      * to register.  This is safe because the thread table is always
      * locked when a thread accesses this field. */
     List *joiners;
+    pid_t pid;
 };
 
 struct new_thread_args
@@ -65,9 +122,9 @@ static struct threadinfo mainthread;
 /* Our key for accessing the (struct gwthread *) we stash in the
  * thread-specific-data area.  This is much more efficient than
  * accessing a global table, which we would have to lock. */
-pthread_key_t tsd_key;
+static pthread_key_t tsd_key;
 
-pthread_mutex_t threadtable_lock;
+static pthread_mutex_t threadtable_lock;
 
 static void lock(void)
 {
@@ -117,6 +174,7 @@ static long fill_threadinfo(pthread_t id, const char *name,
     ti->self = id;
     ti->name = name;
     ti->func = func;
+    ti->pid = -1;
 
     if (pipe(pipefds) < 0) {
         panic(errno, "cannot allocate wakeup pipe for new thread");
@@ -281,6 +339,10 @@ static void *new_thread(void *arg)
         panic(ret, "gwthread-pthread: pthread_setspecific failed");
     }
 
+    p->ti->pid = getpid();
+    debug("gwlib.gwthread", 0, "Thread %ld (%s) maps to pid %ld.",
+          p->ti->number, p->ti->name, (long) p->ti->pid);
+
     (p->func)(p->arg);
 
     lock();
@@ -353,6 +415,7 @@ static void restore_user_signals(sigset_t *old_set)
     }
 }
 
+
 static long spawn_thread(gwthread_func_t *func, const char *name, void *arg)
 {
     int ret;
@@ -542,12 +605,36 @@ long gwthread_self(void)
 {
     struct threadinfo *threadinfo;
     threadinfo = pthread_getspecific(tsd_key);
-    if (threadinfo)
+    if (threadinfo) 
         return threadinfo->number;
     else
         return -1;
 }
 
+/* Return the thread pid of this thread. */
+long gwthread_self_pid(void)
+{
+    struct threadinfo *threadinfo;
+    threadinfo = pthread_getspecific(tsd_key);
+    if (threadinfo && threadinfo->pid != -1) 
+        return (long) threadinfo->pid;
+    else
+        return (long) getpid();
+}
+
+void gwthread_self_ids(long *tid, long *pid)
+{
+    struct threadinfo *threadinfo;
+    threadinfo = pthread_getspecific(tsd_key);
+    if (threadinfo) {
+        *tid = threadinfo->number;
+        *pid = (threadinfo->pid != -1) ? threadinfo->pid : getpid();
+    } else {
+        *tid = -1;
+        *pid = getpid();
+    }
+}
+
 void gwthread_wakeup(long thread)
 {
     unsigned char c = 0;
@@ -581,9 +668,11 @@ int gwthread_pollfd(int fd, int events, double timeout)
 
     pollfd[0].fd = threadinfo->wakefd_recv;
     pollfd[0].events = POLLIN;
+    pollfd[0].revents = 0;
 
     pollfd[1].fd = fd;
     pollfd[1].events = events;
+    pollfd[1].revents = 0;
 
     milliseconds = timeout * 1000;
     if (milliseconds < 0)
