@@ -1,3 +1,59 @@
+/* ====================================================================
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * octstr.c - implementation of Octet strings
  *
@@ -86,13 +142,24 @@ static char is_safe[UCHAR_MAX + 1];
  */
 #define CSTR_TO_LONG(ptr)	(((long) ptr) >> 2)
 
+
+/*
+ * HEX to ASCII preprocessor macro
+ */
+#define H2B(a) (a >= '0' && a <= '9' ? \
+    a - '0' : (a >= 'a' && a <= 'f' ? \
+        a - 'a' + 10 : (a >= 'A' && a <= 'F' ? a - 'A' + 10 : -1) \
+        ) \
+    )
+
+
 /***********************************************************************
  * Declarations of internal functions. These are defined at the end of
  * the file.
  */
 
 
-static void seems_valid_real(Octstr *ostr, const char *filename, long lineno,
+static void seems_valid_real(const Octstr *ostr, const char *filename, long lineno,
                              const char *function);
 #ifdef NO_GWASSERT
 #define seems_valid(ostr)
@@ -118,6 +185,8 @@ static void octstr_grow(Octstr *ostr, long size)
     size++;   /* make room for the invisible terminating NUL */
 
     if (size > ostr->size) {
+        /* always reallocate in 1kB chunks */
+        size += 1024 - (size % 1024);
         ostr->data = gw_realloc(ostr->data, size);
         ostr->size = size;
     }
@@ -167,14 +236,15 @@ void octstr_shutdown(void)
 }
 
 
-Octstr *octstr_create_real(const char *cstr)
+Octstr *octstr_create_real(const char *cstr, const char *file, long line,
+                           const char *func)
 {
     gw_assert(cstr != NULL);
-    return octstr_create_from_data(cstr, strlen(cstr));
+    return octstr_create_from_data_trace(cstr, strlen(cstr), file, line, func);
 }
 
-
-Octstr *octstr_create_from_data_real(const char *data, long len)
+Octstr *octstr_create_from_data_real(const char *data, long len, const char *file,
+                                     long line, const char *func)
 {
     Octstr *ostr;
 
@@ -182,7 +252,7 @@ Octstr *octstr_create_from_data_real(const char *data, long len)
     if (data == NULL)
         gw_assert(len == 0);
 
-    ostr = gw_malloc(sizeof(*ostr));
+    ostr = gw_malloc_trace(sizeof(*ostr), file, line, func);
     if (len == 0) {
         ostr->len = 0;
         ostr->size = 0;
@@ -190,7 +260,7 @@ Octstr *octstr_create_from_data_real(const char *data, long len)
     } else {
         ostr->len = len;
         ostr->size = len + 1;
-        ostr->data = gw_malloc(ostr->size);
+        ostr->data = gw_malloc_trace(ostr->size, file, line, func);
         memcpy(ostr->data, data, len);
         ostr->data[len] = '\0';
     }
@@ -259,7 +329,7 @@ void octstr_destroy_item(void *os)
 }
 
 
-long octstr_len(Octstr *ostr)
+long octstr_len(const Octstr *ostr)
 {
     if (ostr == NULL)
         return 0;
@@ -268,7 +338,8 @@ long octstr_len(Octstr *ostr)
 }
 
 
-Octstr *octstr_copy_real(Octstr *ostr, long from, long len)
+Octstr *octstr_copy_real(const Octstr *ostr, long from, long len, const char *file, long line,
+                         const char *func)
 {
     seems_valid(ostr);
     gw_assert(from >= 0);
@@ -280,17 +351,19 @@ Octstr *octstr_copy_real(Octstr *ostr, long from, long len)
     if (len > ostr->len - from)
         len = ostr->len - from;
 
-    return octstr_create_from_data(ostr->data + from, len);
+    return octstr_create_from_data_trace(ostr->data + from, len, file,
+                                         line, func);
 }
 
 
 
-Octstr *octstr_duplicate_real(Octstr *ostr)
+Octstr *octstr_duplicate_real(const Octstr *ostr, const char *file, long line,
+                              const char *func)
 {
     if (ostr == NULL)
         return NULL;
     seems_valid(ostr);
-    return octstr_create_from_data(ostr->data, ostr->len);
+    return octstr_create_from_data_trace(ostr->data, ostr->len, file, line, func);
 }
 
 
@@ -318,7 +391,7 @@ Octstr *octstr_cat(Octstr *ostr1, Octstr *ostr2)
 }
 
 
-int octstr_get_char(Octstr *ostr, long pos)
+int octstr_get_char(const Octstr *ostr, long pos)
 {
     seems_valid(ostr);
     if (pos >= ostr->len || pos < 0)
@@ -351,7 +424,7 @@ void octstr_get_many_chars(char *buf, Octstr *ostr, long pos, long len)
 }
 
 
-char *octstr_get_cstr_real(Octstr *ostr, const char *file, long line, 
+char *octstr_get_cstr_real(const Octstr *ostr, const char *file, long line, 
     	    	    	   const char *func)
 {
     if (!ostr)
@@ -380,7 +453,7 @@ void octstr_append_from_hex(Octstr *ostr, char *hex)
 void octstr_binary_to_hex(Octstr *ostr, int uppercase)
 {
     unsigned char *hexits;
-    long i;
+    long i, tmp;
 
     seems_valid(ostr);
     gw_assert(!ostr->immutable);
@@ -394,8 +467,9 @@ void octstr_binary_to_hex(Octstr *ostr, int uppercase)
      * overwriting the data while we read it.  Even the order of
      * the two assignments is important, to get i == 0 right. */
     for (i = ostr->len - 1; i >= 0; i--) {
-        ostr->data[i * 2 + 1] = hexits[ostr->data[i] % 16];
-        ostr->data[i * 2] = hexits[(ostr->data[i] / 16) & 0xf];
+        tmp = i << 1; /* tmp = i * 2; */
+        ostr->data[tmp + 1] = hexits[ostr->data[i] & 0xf];
+        ostr->data[tmp] = hexits[ostr->data[i] >> 4];
     }
 
     ostr->len = ostr->len * 2;
@@ -693,6 +767,38 @@ long octstr_parse_long(long *nump, Octstr *ostr, long pos, int base)
 }
 
 
+long octstr_parse_double(double *nump, Octstr *ostr, long pos)
+{
+    /* strtod wants a char *, and we have to compare the result to
+     * an unsigned char *.  The easiest way to avoid warnings without
+     * introducing typecasts is to use two variables. */
+    char *endptr;
+    unsigned char *endpos;
+    double number;
+
+    seems_valid(ostr);
+    gw_assert(nump != NULL);
+
+    if (pos >= ostr->len) {
+        errno = EINVAL;
+        return -1;
+    }
+
+    errno = 0;
+    number = strtod(ostr->data + pos, &endptr);
+    endpos = endptr;
+    if (errno == ERANGE)
+        return -1;
+    if (endpos == ostr->data + pos) {
+        errno = EINVAL;
+        return -1;
+    }
+
+    *nump = number;
+    return endpos - ostr->data;
+}
+
+
 int octstr_check_range(Octstr *ostr, long pos, long len,
                        octstr_func_t filter)
 {
@@ -737,7 +843,7 @@ void octstr_convert_range(Octstr *ostr, long pos, long len,
 }
 
 
-int octstr_compare(Octstr *ostr1, Octstr *ostr2)
+int octstr_compare(const Octstr *ostr1, const Octstr *ostr2)
 {
     int ret;
     long len;
@@ -769,7 +875,7 @@ int octstr_compare(Octstr *ostr1, Octstr *ostr2)
 }
 
 
-int octstr_case_compare(Octstr *os1, Octstr *os2)
+int octstr_case_compare(const Octstr *os1, const Octstr *os2)
 {
     int c1, c2;
     long i, len;
@@ -815,7 +921,7 @@ int octstr_case_compare(Octstr *os1, Octstr *os2)
 }
 
 
-int octstr_ncompare(Octstr *ostr1, Octstr *ostr2, long n)
+int octstr_ncompare(const Octstr *ostr1, const Octstr *ostr2, long n)
 {
     long len;
 
@@ -836,7 +942,7 @@ int octstr_ncompare(Octstr *ostr1, Octstr *ostr2, long n)
 }
 
 
-int octstr_str_compare(Octstr *ostr, const char *str)
+int octstr_str_compare(const Octstr *ostr, const char *str)
 {
     seems_valid(ostr);
 
@@ -849,7 +955,20 @@ int octstr_str_compare(Octstr *ostr, const char *str)
 }
 
 
-int octstr_search_char(Octstr *ostr, int ch, long pos)
+int octstr_str_ncompare(const Octstr *ostr, const char *str, long n)
+{
+    seems_valid(ostr);
+
+    if (str == NULL)
+        return -1;
+    if (ostr->data == NULL)
+        return 1; /* str grater */
+
+    return strncmp(ostr->data, str, n);
+}
+
+
+int octstr_search_char(const Octstr *ostr, int ch, long pos)
 {
     unsigned char *p;
 
@@ -868,7 +987,25 @@ int octstr_search_char(Octstr *ostr, int ch, long pos)
 }
 
 
-int octstr_search(Octstr *haystack, Octstr *needle, long pos)
+int octstr_search_chars(const Octstr *ostr, const Octstr *chars, long pos)
+{
+    long i, j;
+
+    seems_valid(ostr);
+    seems_valid(chars);
+    gw_assert(pos >= 0);
+
+    for (i = 0; i < octstr_len(chars); i++) {
+	j = octstr_search_char(ostr, octstr_get_char(chars, i), pos);
+	if (j != -1)
+	    return j;
+    }
+
+    return -1;
+}
+
+
+int octstr_search(const Octstr *haystack, const Octstr *needle, long pos)
 {
     int first;
 
@@ -900,7 +1037,7 @@ int octstr_search(Octstr *haystack, Octstr *needle, long pos)
 }
 
 
-int octstr_case_search(Octstr *haystack, Octstr *needle, long pos)
+int octstr_case_search(const Octstr *haystack, const Octstr *needle, long pos)
 {
     long i, j;
     int c1, c2;
@@ -927,6 +1064,33 @@ int octstr_case_search(Octstr *haystack, Octstr *needle, long pos)
     return -1;    
 }
 
+int octstr_case_nsearch(const Octstr *haystack, const Octstr *needle, long pos, long n)
+{
+    long i, j;
+    int c1, c2;
+
+    seems_valid(haystack);
+    seems_valid(needle);
+    gw_assert(pos >= 0);
+
+    /* Always "find" an empty string */
+    if (needle->len == 0)
+        return 0;
+
+    for (i = pos; i <= haystack->len - needle->len && i < n; ++i) {
+        for (j = 0; j < needle->len && j < n; ++j) {
+            c1 = toupper(haystack->data[i + j]);
+            c2 = toupper(needle->data[j]);
+            if (c1 != c2)
+                break;
+        }
+        if (j == needle->len)
+            return i;
+    }
+
+    return -1;
+}
+
 
 int octstr_print(FILE *f, Octstr *ostr)
 {
@@ -1040,8 +1204,11 @@ again:
 }
 
 
-void octstr_insert(Octstr *ostr1, Octstr *ostr2, long pos)
+void octstr_insert(Octstr *ostr1, const Octstr *ostr2, long pos)
 {
+    if (ostr2 == NULL)
+        return;
+
     seems_valid(ostr1);
     seems_valid(ostr2);
     gw_assert(pos <= ostr1->len);
@@ -1235,7 +1402,7 @@ void octstr_append_data(Octstr *ostr, const char *data, long len)
 }
 
 
-void octstr_append(Octstr *ostr1, Octstr *ostr2)
+void octstr_append(Octstr *ostr1, const Octstr *ostr2)
 {
     gw_assert(ostr1 != NULL);
     octstr_insert(ostr1, ostr2, ostr1->len);
@@ -1333,7 +1500,7 @@ error:
 }
 
 
-List *octstr_split_words(Octstr *ostr)
+List *octstr_split_words(const Octstr *ostr)
 {
     unsigned char *p;
     List *list;
@@ -1371,7 +1538,7 @@ List *octstr_split_words(Octstr *ostr)
 }
 
 
-List *octstr_split(Octstr *os, Octstr *sep)
+List *octstr_split(const Octstr *os, const Octstr *sep)
 {
     List *list;
     long next, pos, seplen;
@@ -1398,11 +1565,16 @@ int octstr_item_match(void *item, void *pattern)
 }
 
 
-void octstr_dump(Octstr *ostr, int level)
+int octstr_item_case_match(void *item, void *pattern)
+{
+    return octstr_case_compare(item, pattern) == 0;
+}
+
+void octstr_dump(const Octstr *ostr, int level)
 {
     unsigned char *p, *d, buf[1024], charbuf[256];
     long pos;
-    const int octets_per_line = 8;
+    const int octets_per_line = 16;
     int c, this_line_begins_at;
 
     if (ostr == NULL)
@@ -1499,61 +1671,88 @@ void octstr_dump_short(Octstr *ostr, int level, const char *name)
     octstr_dump(ostr, level + 1);
 }
 
+
 void octstr_url_encode(Octstr *ostr)
 {
-    int i, n, newlen;
-    unsigned char *str, *str2, *hexits;
-    unsigned char c;
-    Octstr *res;
+    long i, n, len = 0;
+    int all_safe;
+    unsigned char c, *str, *str2, *res, *hexits;
+
+    if (ostr == NULL)
+        return;
 
     seems_valid(ostr);
+    gw_assert(!ostr->immutable);
 
     if (ostr->len == 0)
-	return;
+        return;
 
-    res = octstr_create("");
-    str = ostr->data;
-    n = 0;
-    for (i = 0; i < ostr->len; i++)
-	if (!is_safe[*str++])
+    /* calculate new length */
+    for (i = n = 0, str = ostr->data, all_safe = 1; i < ostr->len; i++) {
+        c = *str++;
+	
+        if (c == ' ') {
+            all_safe = 0;
+            continue;
+        }
+
+        if (!is_safe[c]) {
 	    n++;
-    newlen = ostr->len + 2 * n;
-    res->len = newlen;
-    res->size = res->len + 1;
-    res->data = gw_malloc(res->size);
+            all_safe = 0;
+        }
+     }
+
+    if (all_safe) /* we are done, all chars are safe */
+       return;
 
     hexits = "0123456789ABCDEF";
 
-    str = ostr->data;
-    str2 = res->data;
-    for (i = 0; i < ostr->len; i++) {
-	c = *str++;
-	if (!is_safe[c]) {
-	    *str2++ = '%';
-	    *str2++ = hexits[c >> 4 & 0xf];
-	    *str2++ = hexits[c & 0xf];
-	}
-	else if (c == ' ')
-	    *str2++ = '+';
-	else
-	    *str2++ = c;
+    /*
+     * no need to reallocate if n == 0, so we make replace in place.
+     * NOTE: we don't do if (xxx) ... else ... because conditional jump
+     * is not so fast as just compare (alex).
+     */
+    res = str2 = (n ? gw_malloc((len = ostr->len + 2 * n + 1)) : ostr->data);
+
+    for (i = 0, str = ostr->data; i < ostr->len; i++) {
+        c = *str++;
+
+        if (c == ' ') {
+            *str2++ = '+';
+            continue;
+        }
+
+        if (!is_safe[c]) {
+            *str2++ = '%';
+            *str2++ = hexits[c >> 4 & 0xf];
+            *str2++ = hexits[c & 0xf];
+            continue;
+        }
+
+        *str2++ = c;
     }
     *str2 = 0;
-    seems_valid(res);
     
-    octstr_truncate(ostr, 0);
-    octstr_insert(ostr, res, 0);
-    octstr_destroy(res);
+    /* we made replace in place */
+    if (n) {
+        gw_free(ostr->data);
+        ostr->data = res;
+        ostr->size = len;
+        ostr->len = len - 1;
+    }
+
+    seems_valid(ostr);
 }
 
 
 int octstr_url_decode(Octstr *ostr)
 {
-    long value;
     unsigned char *string = ostr->data;
     unsigned char *dptr = ostr->data;
-    unsigned char buf[3];    	/* buffer for strtol conversion */
-    buf[2] = '\0';
+    int code, code2, ret = 0;
+
+    if (ostr == NULL)
+        return 0;
 
     seems_valid(ostr);
     gw_assert(!ostr->immutable);
@@ -1563,39 +1762,40 @@ int octstr_url_decode(Octstr *ostr)
 
     do {
         if (*string == '%') {
-            if (*(string + 1) == '\0' || *(string + 2) == '\0')
-                goto error;
-            buf[0] = *(string + 1);
-            buf[1] = *(string + 2);
-            value = strtol(buf, NULL, 16);
+            if (*(string + 1) == '\0' || *(string + 2) == '\0') {
+                warning(0, "octstr_url_decode: corrupted end-of-string <%s>", string);
+                ret = -1;
+                break;
+            }
 
-            if (value >= 0 && value < 256) {
-                *dptr = (unsigned char)value;
-                string += 3;
-                dptr++;
+            code = H2B(*(string + 1));
+            code2 = H2B(*(string + 2));
+
+            if (code == -1 || code2 == -1) {
+                warning(0, "octstr_url_decode: garbage detected (%c%c%c) skipping.",
+                            *string, *(string + 1), *(string + 2));
+                *dptr++ = *string++;
+                *dptr++ = *string++;
+                *dptr++ = *string++;
+                ret = -1;
                 continue;
             }
-            warning(0, "Garbage encoded (value = %ld)", value);
+
+            *dptr++ = code << 4 | code2;
+            string += 3;
         }
-        if (*string == '+') {
+        else if (*string == '+') {
             *dptr++ = ' ';
             string++;
         } else
             *dptr++ = *string++;
     } while (*string); 	/* we stop here because it terimates encoded string */
-    *dptr = '\0';
 
+    *dptr = '\0';
     ostr->len = (dptr - ostr->data);
 
     seems_valid(ostr);
-    return 0;
-
-error:
-    *dptr = '\0';
-    ostr->len = (dptr - ostr->data);
-    warning(0, "octstr_url_decode: corrupted end-of-string <%s>", string);
-    seems_valid(ostr);
-    return -1;
+    return ret;
 }
 
 
@@ -1775,12 +1975,9 @@ long octstr_extract_uintvar(Octstr *ostr, unsigned long *value, long pos)
 void octstr_append_decimal(Octstr *ostr, long value)
 {
     char tmp[128];
-    Octstr *ostmp;
 
     sprintf(tmp, "%ld", value);
-    ostmp = octstr_create(tmp);
-    octstr_append(ostr, ostmp);
-    octstr_destroy(ostmp);
+    octstr_append_cstr(ostr, tmp);
 }
 
 
@@ -1999,14 +2196,24 @@ static void convert(Octstr *os, struct format *format, const char **fmt,
         new = octstr_duplicate(va_arg(VALST(args), Octstr *));
         if (!new)
             new = octstr_imm("(null)");
-	octstr_url_encode(new);
-	/*
-	 * note: we use blind truncate - encoded character can get cut half-way.
+        octstr_url_encode(new);
+        /*
+         * note: we use blind truncate - encoded character can get cut half-way.
          */
         if (format->has_prec)
             octstr_truncate(new, format->prec);
         break;
 
+    case 'H':
+        new = octstr_duplicate(va_arg(VALST(args), Octstr *));
+        if (!new)
+            new = octstr_imm("(null)");
+        /* upper case */
+        octstr_binary_to_hex(new, 1);
+        if (format->has_prec)
+            octstr_truncate(new, (format->prec % 2 ? format->prec - 1 : format->prec));
+        break;
+
     case '%':
     	new = octstr_imm("%");
     	break;
@@ -2111,7 +2318,7 @@ unsigned long octstr_hash_key(Octstr *ostr)
  * Local functions.
  */
 
-static void seems_valid_real(Octstr *ostr, const char *filename, long lineno,
+static void seems_valid_real(const Octstr *ostr, const char *filename, long lineno,
                              const char *function)
 {
     gw_assert(immutables_init);
@@ -2181,3 +2388,198 @@ octstr_recode (Octstr *tocode, Octstr *fromcode, Octstr *orig)
 
     return resultcode;
 }
+
+void octstr_strip_char(Octstr *text, char ch)
+{
+    int start = 0;
+
+    seems_valid(text);
+    gw_assert(!text->immutable);
+
+    /* Remove char from the beginning of the text */
+    while ((ch == octstr_get_char(text, start)) &&
+           start <= octstr_len(text))
+        start ++;
+
+    if (start > 0)
+        octstr_delete(text, 0, start);
+
+    seems_valid(text);
+}
+
+int octstr_isnum(Octstr *ostr1)
+{
+    int start = 0;
+    char c;
+
+    seems_valid(ostr1);
+    while (start < octstr_len(ostr1)) {
+        c = octstr_get_char(ostr1, start);
+        if (!isdigit(c) && (c!='+'))
+            return 0;
+        start++;
+    }
+    return 1;
+}
+
+void octstr_replace(Octstr *haystack, Octstr *needle, Octstr *repl)
+{
+    int p = 0;
+    long len, repl_len;
+
+    len = octstr_len(needle);
+    repl_len = octstr_len(repl);
+
+    while ((p = octstr_search(haystack, needle, p)) != -1) {
+        octstr_delete(haystack, p, len);
+        octstr_insert(haystack, repl, p);
+        p += repl_len;
+    }
+}
+
+int octstr_symbolize(Octstr *ostr)
+{
+    long len, i;
+
+    seems_valid(ostr);
+    gw_assert(!ostr->immutable);
+
+    if (ostr->len == 0)
+        return 0;
+
+    /* Check if it's in the right format */
+    if (!octstr_check_range(ostr, 0, ostr->len, gw_isxdigit))
+        return -1;
+
+    len = ostr->len + (ostr->len/2);
+    octstr_grow(ostr, ostr->len * 2);
+
+    for (i = 0; i < len; i += 3)
+        octstr_insert_data(ostr, i, "%", 1);
+
+    return 1;
+}
+
+void octstr_delete_matching(Octstr *haystack, Octstr *needle)
+{
+    int p = -1;
+    long len;
+
+    seems_valid(haystack);
+    seems_valid(needle);
+    gw_assert(!haystack->immutable);
+    len = octstr_len(needle);
+
+    while ((p = octstr_search(haystack, needle, p + 1)) != -1) {
+        octstr_delete(haystack, p, len);
+        p -= len;
+    }
+}
+ 
+int octstr_is_all_hex(Octstr *os)      
+{
+    long len, i;
+    int ch;
+
+    seems_valid(os);
+    len = octstr_len(os);
+    for (i = 0; i < len; ++i) {
+        ch = octstr_get_char(os, i);
+        if (!gw_isxdigit(ch))
+            return 0;
+    }
+
+    return 1;
+}
+
+/*
+ * function octstr_convert_to_html_entities()
+ *      make data HTML safe by converting appropriate characters to HTML entities
+ * Input: data to be inserted in HTML
+ **/
+void octstr_convert_to_html_entities(Octstr* input)
+{
+    int i;
+
+    for (i = 0; i < octstr_len(input); ++i) {
+        switch (octstr_get_char(input, i)) {
+#define ENTITY(a,b) \
+    case a: \
+    octstr_delete(input, i, 1); \
+    octstr_insert(input, octstr_imm("&" b ";"), i); \
+    i += sizeof(b); break;
+#include "gwlib/html-entities.def"
+#undef ENTITY
+        }
+    }
+}
+
+/*
+ * This function is meant to find html entities in an octstr.
+ * The html-entities.def file must be sorted alphabetically for
+ * this function to work (according to current Locale in use).
+*/
+static int octstr_find_entity(Octstr* input, int startfind, int endfind)
+{
+#define ENTITY(a,b) { a, b },
+    struct entity_struct {
+        int entity;
+        char *entity_str;
+    };
+    const struct entity_struct entities[] = {
+#include "html-entities.def"
+        { -1, "" } /* pivot */
+    };
+#undef ENTITY
+    int center;         /* position in table that we are about to compare */
+    int matchresult;    /* result of match agains found entity name. indicates less, equal or greater */
+
+    if (endfind == 0) {
+        /* when calling this function we do not (nor even want to) know the
+         * sizeof(entities). Hence this check. */
+        endfind = (sizeof(entities) / sizeof(struct entity_struct)) - 1;
+    }
+    center = startfind + ((endfind - startfind) / 2);
+    matchresult = octstr_str_compare(input, entities[center].entity_str);
+    if (matchresult == 0) {
+        return entities[center].entity;
+    }
+    if (endfind - startfind <= 1) {
+        /* we are at the end of our results */
+        return -1;
+    }
+    if  (matchresult < 0) {
+        /* keep searching in first part of the table */
+        return octstr_find_entity(input, startfind, center);
+    } else {
+        /* keep searching in last part of the table */
+        return octstr_find_entity(input, center, endfind);
+    }
+}
+
+/*
+ * function octstr_convert_from_html_entities()
+ *   convert HTML safe data back to binary data by replacing HTML entities with their
+ *   respective character values
+ * Input: data to be inserted in HTML
+ **/
+void octstr_convert_from_html_entities(Octstr* input)
+{
+    int startpos = 0, endpos;
+    int entity;
+    Octstr *match;
+
+    while ((startpos = octstr_search_char(input, '&', startpos)) != -1) {
+        endpos = octstr_search_char(input, ';', startpos + 1);
+        if (endpos >= 0) {
+            match = octstr_copy(input, startpos + 1, endpos - startpos - 1);
+            entity = octstr_find_entity(match, 0, 0);
+            if (entity >= 0) {
+                octstr_delete(input, startpos, endpos - startpos + 1);
+                octstr_insert_char(input, startpos, entity);
+            }
+            octstr_destroy(match);
+        }
+        startpos++;
+    }
+}
