@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * fakewap.c - simulate wap clients talking directly to wap gw.
  *
@@ -86,7 +142,7 @@ where options are:\n\
 -p port		port number of gateway (default: 9201)\n\
 -m max		maximum number of requests fakewap will make (default: 1)\n\
 -i interval	interval between requests (default: 1.0 seconds)\n\
--c threads	number concurrent of clients simulated (default: 1)\n\
+-c threads	number of concurrent clients simulated (default: 1)\n\
 -V protoversion	protocol version field, as an integer (default: 0)\n\
 -T pdu-type	PDU type, as an integer (default: 1)\n\
 -t tcl		transaction class, as an integer (default: 2)\n\
@@ -96,7 +152,7 @@ where options are:\n\
                 (default: off)\n\
 -d difference	difference between successive tid numbers (default: 1)\n\
 -F		Accept failure and continue rather than exiting\n\
--w		Write/print recieved data (experimental)\n\
+-w		Write/print received data (experimental)\n\
 \n\
 The urls are fetched in random order.\n\
 ";
@@ -157,8 +213,16 @@ int test_separation = 0;
  * command line argument. WSP_Concat is a concatenation of WTP_Ack and 
  * WSP_Disconnect PDUs.
  */
-unsigned char WSP_Connect[] = {0x06, 0x00, 0x00, 0x00, 0x01, 0x10, 0x00, 
-				0x00 };
+unsigned char WSP_Connect[] = {0x06, 0x00, 0x00, 0x00, 
+				/* WSP part */
+				0x01, /* PDU type */
+				0x10, /* Version 1.0 */
+				0x00, /* Capability length */
+				0x02, /* Headers length = 2*/
+				/* Capabilities */
+				/* Headers */
+				0x80, 0x80 /* Accept: *\* */
+				};
 unsigned char WSP_ConnectReply[] = {0x16, 0x80, 0x00, 0x02 };
 unsigned char WTP_Ack[] =          {0x18, 0x00, 0x00 };
 unsigned char WTP_TidVe[] =        {0x1C, 0x00, 0x00 };
@@ -415,6 +479,9 @@ wap_msg_recv( int fd, const char * hdr, int hdr_len,
             else if (GET_WTP_PDU_TYPE(msg) == WTP_PDU_ABORT) {
                 print_msg( "Received WTP Abort", msg, msg_len );
             }
+            else if (GET_WTP_PDU_TYPE(msg) == WTP_PDU_RESULT) {
+               break;
+            }
             else {
                 print_msg( "Received unexpected message", msg, msg_len );
             }
