@@ -1,7 +1,64 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * wsp_session.c - Implement WSP session oriented service
  *
  * Lars Wirzenius
+ * Stipe Tolj
  */
 
 
@@ -87,6 +144,7 @@ static char *state_name(WSPState state);
 static unsigned long next_wsp_session_id(void);
 
 static List *make_capabilities_reply(WSPMachine *m);
+static List *make_reply_headers(WSPMachine *m);
 static Octstr *make_connectreply_pdu(WSPMachine *m);
 static Octstr *make_resume_reply_pdu(WSPMachine *m, List *headers);
 static WSP_PDU *make_confirmedpush_pdu(WAPEvent *e);
@@ -123,7 +181,8 @@ static void confirm_push(WSPPushMachine *machine);
 
 static void main_thread(void *);
 static int id_belongs_to_session (void *, void *);
-
+static int wsp_encoding_string_to_version(Octstr *enc);
+static Octstr *wsp_encoding_version_to_string(int version);
 
 
 /***********************************************************************
@@ -947,48 +1006,80 @@ static List *make_capabilities_reply(WSPMachine *m) {
 }
 
 
-static Octstr *make_connectreply_pdu(WSPMachine *m) {
-	WSP_PDU *pdu;
-	Octstr *os;
-	List *caps;
+static List *make_reply_headers(WSPMachine *m)
+{
+    List *headers;
+    Octstr *encoding_version;
+
+    /* Add all server wsp level hop-by-hop headers. Currently only 
+     * Encoding-Version, as defined by wsp, chapter 8.4.2.70. 
+     * What headers belong to which version is defined in appendix A,
+     * table 39.. 
+    encoding_version = request_version = NULL;
+     * Essentially, if the client sends us an Encoding-Version
+     * higher than ours (1.3) we send our version number to it,
+     * if it is lower, we left version number intact. */
+    /* First the case that we have no Encoding-Version header at all. 
+     * This case we must assume that the client supports version 1.2
+     * or lower. */
+
+    headers = http_create_empty_headers();
+    encoding_version = wsp_encoding_version_to_string(m->encoding_version);
+    http_header_add(headers, "Encoding-Version", octstr_get_cstr(encoding_version));
+    octstr_destroy(encoding_version);
+
+    return headers;
+}
+
+static Octstr *make_connectreply_pdu(WSPMachine *m) 
+{
+    WSP_PDU *pdu;
+    Octstr *os;
+    List *caps;
+    List *reply_headers;
 	
-	pdu = wsp_pdu_create(ConnectReply);
+    pdu = wsp_pdu_create(ConnectReply);
 
-	pdu->u.ConnectReply.sessionid = m->session_id;
+    pdu->u.ConnectReply.sessionid = m->session_id;
 
-	caps = make_capabilities_reply(m);
-	pdu->u.ConnectReply.capabilities = wsp_cap_pack_list(caps);
-	wsp_cap_destroy_list(caps);
-	pdu->u.ConnectReply.headers = NULL;
+    caps = make_capabilities_reply(m);
+    pdu->u.ConnectReply.capabilities = wsp_cap_pack_list(caps);
+    wsp_cap_destroy_list(caps);
+
+    reply_headers = make_reply_headers(m);
+    pdu->u.ConnectReply.headers = 
+        wsp_headers_pack(reply_headers, 0, m->encoding_version);
+    http_destroy_headers(reply_headers);
 	
-	os = wsp_pdu_pack(pdu);
-	wsp_pdu_destroy(pdu);
+    os = wsp_pdu_pack(pdu);
+    wsp_pdu_destroy(pdu);
 
-	return os;
+    return os;
 }
 
 
-static Octstr *make_resume_reply_pdu(WSPMachine *m, List *headers) {
-	WSP_PDU *pdu;
-	Octstr *os;
+static Octstr *make_resume_reply_pdu(WSPMachine *m, List *headers) 
+{
+    WSP_PDU *pdu;
+    Octstr *os;
 
-	pdu = wsp_pdu_create(Reply);
+    pdu = wsp_pdu_create(Reply);
 
-	/* Not specified for Resume replies */
-	pdu->u.Reply.status = wsp_convert_http_status_to_wsp_status(HTTP_OK);
-	if (headers == NULL) {
-		headers = http_create_empty_headers();
-		pdu->u.Reply.headers = wsp_headers_pack(headers, 1);
-		http_destroy_headers(headers);
-	} else {
-		pdu->u.Reply.headers = wsp_headers_pack(headers, 1);
-	}
-	pdu->u.Reply.data = octstr_create("");
+    /* Not specified for Resume replies */
+    pdu->u.Reply.status = wsp_convert_http_status_to_wsp_status(HTTP_OK);
+    if (headers == NULL) {
+        headers = http_create_empty_headers();
+        pdu->u.Reply.headers = wsp_headers_pack(headers, 1, m->encoding_version);
+        http_destroy_headers(headers);
+    } else {
+        pdu->u.Reply.headers = wsp_headers_pack(headers, 1, m->encoding_version);
+    }
+    pdu->u.Reply.data = octstr_create("");
 
-	os = wsp_pdu_pack(pdu);
-	wsp_pdu_destroy(pdu);
+    os = wsp_pdu_pack(pdu);
+    wsp_pdu_destroy(pdu);
 
-	return os;
+    return os;
 }
 
 static WSP_PDU *make_confirmedpush_pdu(WAPEvent *e)
@@ -1002,11 +1093,11 @@ static WSP_PDU *make_confirmedpush_pdu(WAPEvent *e)
  */
         if (e->u.S_ConfirmedPush_Req.push_headers == NULL) {
 	    headers = http_create_empty_headers();
-            pdu->u.ConfirmedPush.headers = wsp_headers_pack(headers, 1);
+            pdu->u.ConfirmedPush.headers = wsp_headers_pack(headers, 1, WSP_1_2);
             http_destroy_headers(headers);
         } else
             pdu->u.ConfirmedPush.headers = 
-                wsp_headers_pack(e->u.S_ConfirmedPush_Req.push_headers, 1);
+                wsp_headers_pack(e->u.S_ConfirmedPush_Req.push_headers, 1, WSP_1_2);
    
         if (e->u.S_ConfirmedPush_Req.push_body == NULL)
 	    pdu->u.ConfirmedPush.data = octstr_create("");
@@ -1028,11 +1119,11 @@ static WSP_PDU *make_push_pdu(WAPEvent *e)
  */
         if (e->u.S_Push_Req.push_headers == NULL) {
 	    headers = http_create_empty_headers();
-            pdu->u.Push.headers = wsp_headers_pack(headers, 1);
+            pdu->u.Push.headers = wsp_headers_pack(headers, 1, WSP_1_2);
             http_destroy_headers(headers);
         } else
             pdu->u.Push.headers = 
-                wsp_headers_pack(e->u.S_Push_Req.push_headers, 1);
+                wsp_headers_pack(e->u.S_Push_Req.push_headers, 1, WSP_1_2);
    
         if (e->u.S_Push_Req.push_body == NULL)
 	    pdu->u.Push.data = octstr_create("");
@@ -1320,6 +1411,7 @@ static void abort_methods(WSPMachine *sm, long reason) {
 	wap_event_destroy(ab);
 }
 
+
 static void abort_pushes(WSPMachine *sm, long reason)
 {
         WAPEvent *ab;
@@ -1344,6 +1436,7 @@ WSPMachine *find_session_machine_by_id (int id) {
 	return list_search(session_machines, &id, id_belongs_to_session);
 }
 
+
 static int id_belongs_to_session (void *wsp_ptr, void *pid) {
 	WSPMachine *wsp;
 	int *id;
@@ -1356,10 +1449,57 @@ static int id_belongs_to_session (void *wsp_ptr, void *pid) {
 }
 
 
+static int wsp_encoding_string_to_version(Octstr *enc) 
+{
+    int v;
+    
+    /* default will be WSP 1.2, as defined by WAPWSP */
+    v = WSP_1_2;    
 
+    if (octstr_compare(enc, octstr_imm("1.1")) == 0) {
+        v = WSP_1_1;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.2")) == 0) {
+        v = WSP_1_2;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.3")) == 0) {
+        v = WSP_1_3;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.4")) == 0) {
+        v = WSP_1_4;
+    }
+    else if (octstr_compare(enc, octstr_imm("1.5")) == 0) {
+        v = WSP_1_5;
+    }
 
+    return v;
+}
 
-
-
-
+static Octstr *wsp_encoding_version_to_string(int version) 
+{
+    Octstr *os;
+    
+    switch (version) {
+        case WSP_1_1:
+            os = octstr_create("1.1");
+            break;
+        case WSP_1_2:
+            os = octstr_create("1.2");
+            break;
+        case WSP_1_3:
+            os = octstr_create("1.3");
+            break;
+        case WSP_1_4:
+            os = octstr_create("1.4");
+            break;
+        case WSP_1_5:
+            os = octstr_create("1.5");
+            break;
+        default:
+            os = octstr_create("1.2");
+            break;
+    }
+    
+    return os;
+}
 
